Package: moewishart
Title: Mixture-of-Experts Wishart Models for Covariance Data
Version: 1.2
Description: Methods for maximum likelihood and Bayesian estimation for the Wishart mixture model and the mixture-of-experts Wishart (MoE-Wishart) model. The package provides four inference algorithms for these models, each implemented using the expectation–maximization (EM) algorithm for maximum likelihood estimation and a fully Bayesian approach via Gibbs-within-Metropolis–Hastings sampling.
Authors@R: c(person("The Tien", "Mai", role=c("aut")),
                person("Zhi", "Zhao", role=c("aut","cre"), email = "zhi.zhao@medisin.uio.no"))
Maintainer: Zhi Zhao <zhi.zhao@medisin.uio.no>
URL: https://github.com/zhizuio/moewishart
BugReports: https://github.com/zhizuio/moewishart/issues
License: GPL-3
VignetteBuilder: knitr
Depends: R (>= 4.1.0)
Encoding: UTF-8
Imports: loo, stats, utils, graphics
Suggests: rmarkdown, knitr
LazyData: true
NeedsCompilation: no
Config/roxygen2/version: 8.0.0
Packaged: 2026-06-04 22:08:26 UTC; zhiz
Author: The Tien Mai [aut],
  Zhi Zhao [aut, cre]
Repository: CRAN
Date/Publication: 2026-06-05 06:20:18 UTC
Built: R 4.5.2; ; 2026-06-05 09:52:29 UTC; unix

# Generated by roxygen2: do not edit by hand

export(computeIC)
export(dWishart)
export(lmvgamma)
export(mixturewishart)
export(moewishart)
export(plotMCMC)
export(rdirichlet)
export(sampleIW)
export(simData)
importFrom(graphics,legend)
importFrom(graphics,matplot)
importFrom(loo,loo)
importFrom(stats,kmeans)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,rWishart)
importFrom(stats,rbinom)
importFrom(stats,rexp)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,toeplitz)
importFrom(utils,combn)
importFrom(utils,tail)

[![CRAN
status](https://www.r-pkg.org/badges/version/moewishart)](https://cran.r-project.org/package=moewishart)
[![r-universe](https://zhizuio.r-universe.dev/badges/moewishart)](https://zhizuio.r-universe.dev/moewishart)
[![R-CMD-check](https://github.com/zhizuio/moewishart/workflows/R-CMD-check/badge.svg)](https://github.com/zhizuio/moewishart/actions)
[![License](https://img.shields.io/badge/License-GPLv3-brightgreen.svg)](https://www.gnu.org/licenses/gpl-3.0)



This R-package `moewishart` provides maximum likelihood estimation (MLE) and Bayesian estimation for the **Wishart mixture model** and the **Wishart mixture-of-experts** (**MoE-Wishart**) model. 
It implements four different inference algorithms for the two model:

- mixture model of Wishart distributions:
    - EM algorithm for finding the MLE;
    - Bayesian approach using Gibbs-within-MH sampling algorithm.
- Mixture-of-Expert model, in which the gating probabilities depend on covariates:
   - EM-MoE algorithm for finding the MLE;
   - Bayesian-MoE approach using a Gibbs-within-MH sampling algorithm.  


## Installation

Install the latest released version from [CRAN](https://CRAN.R-project.org/package=moewishart):


``` r
install.packages("BayesSUR")
```

Install the latest development version from [GitHub](https://github.com/zhizuio/moewishart):


``` r
# library("devtools")
devtools::install_github("zhizuio/moewishart")
```

## Example


<br> 

Data simulation from a MoE-Wishart model:

- Sample size $n = 200$
- Dimension of the Wishart distribution $p = 2$
- Number of latent components $K = 3$
- $q=3$ gating covariates $\mathbf X = [x_{ij}] \in \mathbb R^{n\times q}$, $x_{ij}\sim\text{N}(0,1)$, $i=1,...,n$, $j=1,...,q$
- Fixed covariate effects $\boldsymbol\beta=[\boldsymbol\beta_1,...,\boldsymbol\beta_K] \in \mathbb R^{q\times K}$, with $\boldsymbol\beta_{K}=0$
- Probabilities of subpopulations $\boldsymbol\pi = [\pi_{ik}] \in \mathbb R^{n\times q}$, $\pi_{ik} = \exp(\mathbf X_i\boldsymbol\beta_k) / \sum_{l=1}^K\exp(\mathbf X_i\boldsymbol\beta_l)$, $k=1,...,K$
- Degrees of freedom $\boldsymbol\nu = (\nu_1,\nu_2,\nu_3) = (8, 12, 3)$
- Scale matrices of the Wishart distribution $\Sigma_1$, $\Sigma_2$, $\Sigma_3 \in \mathbb R^{p\times p}$
- Data $S_i \sim \pi_{i1}\text{Wishart}(\nu_1, \Sigma_1) + \pi_{i2}\text{Wishart}(\nu_2, \Sigma_2) + \pi_{i3}\text{Wishart}(\nu_3, \Sigma_3)$


### 1. Working model: Bayesian MoE-Wishart model


``` r
library(moewishart)

n <- 200 # number of subjects
p <- 2 # dimension of covariance matrix
set.seed(123) # fix coefficients of underlying MoE model
Xq <- 3
K <- 3
betas <- matrix(runif(Xq * K, -2, 2), nrow = Xq, ncol = K)
betas[, K] <- 0

# simulate data
dat <- simData(n, p,
  Xq = 3, K = 3, betas = betas,
  pis = c(0.35, 0.40, 0.25),
  nus = c(8, 12, 3)
)

# fit Bayesian MoE-Wishart model
set.seed(123)
fit <- moewishart(
  dat$S,
  X = cbind(1, dat$X), K = 3,
  mh_sigma = c(0.2, 0.1, 0.2), # RW-MH variances (length K)
  mh_beta = c(0.3, 0.3), # RW-MH variances (length K-1)
  niter = 3000, burnin = 1000
)
```

<br> 

Posterior means for degrees of freedom (DoF) of Wishart distributions:


``` r
burnin <- 1000
nu_mcmc <- fit$nu[-c(1:burnin), ]
colMeans(nu_mcmc)
```

```
## [1]  8.574911 14.397351  3.310689
```

<br> 
True DoF:


``` r
dat$nu # true nu
```

```
## [1]  8 12  3
```

<br> 
Posterior means for scale matrices of Wishart distributions:


``` r
MoE_Sigma <- Reduce("+", fit$Sigma) / length(fit$Sigma)
MoE_Sigma
```

```
## , , 1
## 
##           [,1]      [,2]
## [1,] 0.5197160 0.2103881
## [2,] 0.2103881 0.7470847
## 
## , , 2
## 
##           [,1]      [,2]
## [1,] 1.7637949 0.5540576
## [2,] 0.5540576 1.3244947
## 
## , , 3
## 
##            [,1]       [,2]
## [1,]  4.1115070 -0.1267705
## [2,] -0.1267705  3.0385263
```

<br> 
Posterior means for gating coefficients:


``` r
beta_mcmc <- fit$Beta_samples[-c(1:burnin), , ]
apply(beta_mcmc, c(2, 3), mean)
```

```
##            [,1]        [,2] [,3]
## [1,] -0.3656861 -0.08024419    0
## [2,] -0.9526224  2.24956385    0
## [3,]  1.7609922  2.40287152    0
## [4,] -0.4953755 -2.56072719    0
```


### 2. Working model: Bayesian Wishart mixture model


``` r
# fit Bayesian Wishart mixture model
set.seed(123)
fit2 <- mixturewishart(
  dat$S,
  K = 3,
  mh_sigma = c(0.2, 0.1, 0.2), # RW-MH variances
  niter = 3000, burnin = 1000
)
```

<br> 
Posterior means for subpopulation probabilities:


``` r
colMeans(fit2$pi[-c(1:burnin), ])
```

```
## [1] 0.2690425 0.5088864 0.2220712
```

<br> 

Posterior means for DoF of Wishart distributions:


``` r
colMeans(fit2$nu[-c(1:burnin), ])
```

```
## [1]  7.986113 12.153338  3.284252
```


### 3. Working model: MoE-Wishart model via EM algorithm


``` r
# fit MoE-Wishart model via EM alg.
set.seed(123)
fit3 <- moewishart(
  dat$S,
  X = cbind(1, dat$X), K = 3,
  method = "em",
  niter = 3000
)
```

```
## Iter   1  loglik = -2079.322610
## Iter   2  loglik = -1998.694495
## Iter   3  loglik = -1985.659443
## Iter   4  loglik = -1947.223842
## Iter   5  loglik = -1899.666938
## Iter   6  loglik = -1878.233062
## Iter   7  loglik = -1861.702657
## Iter   8  loglik = -1851.548347
## Iter   9  loglik = -1847.342510
## Iter  10  loglik = -1845.497390
## Iter  11  loglik = -1844.693281
## Iter  12  loglik = -1844.360139
## Iter  13  loglik = -1844.220804
## Iter  14  loglik = -1844.160033
## Iter  15  loglik = -1844.132434
## Iter  16  loglik = -1844.119493
## Iter  17  loglik = -1844.113254
## Iter  18  loglik = -1844.110247
## Iter  19  loglik = -1844.108705
## Iter  20  loglik = -1844.107915
## Iter  21  loglik = -1844.107527
## Iter  22  loglik = -1844.107312
## Iter  23  loglik = -1844.107207
## Iter  24  loglik = -1844.107148
## Iter  25  loglik = -1844.107116
## Iter  26  loglik = -1844.107098
## Iter  27  loglik = -1844.107088
## Iter  28  loglik = -1844.107082
## Iter  29  loglik = -1844.107080
## Iter  30  loglik = -1844.107079
## Iter  31  loglik = -1844.107077
## Iter  32  loglik = -1844.107077
## Converged by loglik tolerance.
```

<br> 

EM estimates for DoF of Wishart distributions:


``` r
fit3$nu
```

```
## [1]  7.515417 13.987158  3.274665
```

<br> 
EM estimates for Wishart scale matrices:


``` r
fit3$Sigma
```

```
## [[1]]
##           [,1]      [,2]
## [1,] 0.5591113 0.2324429
## [2,] 0.2324429 0.8148737
## 
## [[2]]
##           [,1]      [,2]
## [1,] 1.7665723 0.5567668
## [2,] 0.5567668 1.3336367
## 
## [[3]]
##            [,1]       [,2]
## [1,]  4.3139885 -0.1886288
## [2,] -0.1886288  3.0983710
```

<br> 
EM estimates for gating coefficients:


``` r
fit3$Beta
```

```
##             comp1      comp2 comp3
## [1,] -0.006270492  0.1302039     0
## [2,] -0.798302303  2.0340525     0
## [3,]  1.598530103  2.2399293     0
## [4,] -0.510585695 -2.3465248     0
```


### 4. Working model: Wishart mixture model via EM algorithm


``` r
# fit Wishart mixture model via EM alg.
set.seed(123)
fit4 <- mixturewishart(
  dat$S,
  K = 3,
  method = "em",
  niter = 3000
)
```

```
## Running 3 initialization restarts...
##   -> Restart 1: Loglik = -2011.91
##   -> Restart 2: Loglik = -1989.16
##   -> Restart 3: Loglik = -1989.16
## Iter  10 | Loglik: -1953.0326 | Nu: 5, 7.26, 5
## Iter  20 | Loglik: -1952.7924 | Nu: 5.15, 7.05, 5.21
## Iter  30 | Loglik: -1930.6036 | Nu: 4.21, 7.13, 8.27
## Iter  40 | Loglik: -1902.9306 | Nu: 3.08, 7.78, 10.79
## Iter  50 | Loglik: -1902.6288 | Nu: 3, 7.76, 11.11
## Iter  60 | Loglik: -1902.6133 | Nu: 2.99, 7.71, 11.21
## Iter  70 | Loglik: -1902.6102 | Nu: 2.99, 7.69, 11.26
## Iter  80 | Loglik: -1902.6095 | Nu: 2.99, 7.68, 11.28
## Iter  90 | Loglik: -1902.6093 | Nu: 3, 7.68, 11.3
## Iter 100 | Loglik: -1902.6092 | Nu: 3, 7.68, 11.3
## Iter 110 | Loglik: -1902.6092 | Nu: 3, 7.68, 11.31
## Converged at iteration 116
```

<br> 
EM estimates for DoF of Wishart distributions:


``` r
fit4$nu
```

```
## [1]  2.995383  7.682819 11.309040
```

<br> 
EM estimate for Wishart scale matrices:


``` r
fit4$Sigma
```

```
## [[1]]
##           [,1]     [,2]
## [1,]  4.048859 -1.10103
## [2,] -1.101030  2.79641
## 
## [[2]]
##           [,1]      [,2]
## [1,] 0.5582012 0.2515063
## [2,] 0.2515063 0.8151752
## 
## [[3]]
##           [,1]      [,2]
## [1,] 2.0930529 0.6273737
## [2,] 0.6273737 1.5706730
```

<img src="man/figures/logo.png" align="right" width="160"/>

[![CRAN](http://www.r-pkg.org/badges/version/moewishart)](https://cran.r-project.org/package=moewishart)
[![r-universe](https://zhizuio.r-universe.dev/badges/moewishart)](https://zhizuio.r-universe.dev/moewishart)
[![R-CMD-check](https://github.com/zhizuio/moewishart/workflows/R-CMD-check/badge.svg)](https://github.com/zhizuio/moewishart/actions)
[![License](https://img.shields.io/badge/License-GPLv3-brightgreen.svg)](https://www.gnu.org/licenses/gpl-3.0)

# Mixture-of-Experts Wishart models

This R-package `moewishart` provides maximum likelihood estimation (MLE)
and Bayesian estimation for the **Wishart mixture model** and the
**Wishart mixture-of-experts** (**MoE-Wishart**) model. It implements
four different inference algorithms for the two model:

- mixture model of Wishart distributions:
  - EM algorithm for finding the MLE;
  - Bayesian approach using Gibbs-within-MH sampling algorithm.
- Mixture-of-Expert model, in which the gating probabilities depend on
  covariates:
  - EM-MoE algorithm for finding the MLE;
  - Bayesian-MoE approach using a Gibbs-within-MH sampling algorithm.

## Installation

Install the latest released version from
[CRAN](https://CRAN.R-project.org/package=moewishart):

``` r
install.packages("moewishart")
```

Install the latest development version from
[GitHub](https://github.com/zhizuio/moewishart):

``` r
# library("devtools")
devtools::install_github("zhizuio/moewishart")
```

## Example

<br>

Data simulation from a MoE-Wishart model:

- Sample size $n = 200$
- Dimension of the Wishart distribution $p = 2$
- Number of latent components $K = 3$
- $q=3$ gating covariates
  $\mathbf X = [x_{ij}] \in \mathbb R^{n\times q}$,
  $x_{ij}\sim\text{N}(0,1)$, $i=1,...,n$, $j=1,...,q$
- Fixed covariate effects
  $\boldsymbol\beta=[\boldsymbol\beta_1,...,\boldsymbol\beta_K] \in \mathbb R^{q\times K}$,
  with $\boldsymbol\beta_{K}=0$
- Probabilities of subpopulations
  $\boldsymbol\pi = [\pi_{ik}] \in \mathbb R^{n\times q}$,
  $\pi_{ik} = \exp(\mathbf X_i\boldsymbol\beta_k) / \sum_{l=1}^K\exp(\mathbf X_i\boldsymbol\beta_l)$,
  $k=1,...,K$
- Degrees of freedom $\boldsymbol\nu = (\nu_1,\nu_2,\nu_3) = (8, 12, 3)$
- Scale matrices of the Wishart distribution $\Sigma_1$, $\Sigma_2$,
  $\Sigma_3 \in \mathbb R^{p\times p}$
- Data
  $S_i \sim \pi_{i1}\text{Wishart}(\nu_1, \Sigma_1) + \pi_{i2}\text{Wishart}(\nu_2, \Sigma_2) + \pi_{i3}\text{Wishart}(\nu_3, \Sigma_3)$

### 1. Working model: Bayesian MoE-Wishart model

``` r
library(moewishart)

n <- 200 # number of subjects
p <- 2 # dimension of covariance matrix
set.seed(123) # fix coefficients of underlying MoE model
Xq <- 3
K <- 3
betas <- matrix(runif(Xq * K, -2, 2), nrow = Xq, ncol = K)
betas[, K] <- 0

# simulate data
dat <- simData(n, p,
  Xq = 3, K = 3, betas = betas,
  pis = c(0.35, 0.40, 0.25),
  nus = c(8, 12, 3)
)

# fit Bayesian MoE-Wishart model
set.seed(123)
fit <- moewishart(
  dat$S,
  X = cbind(1, dat$X), K = 3,
  mh_sigma = c(0.2, 0.1, 0.2), # RW-MH variances (length K)
  mh_beta = c(0.3, 0.3), # RW-MH variances (length K-1)
  niter = 3000, burnin = 1000
)
```

<br>

Posterior means for degrees of freedom (DoF) of Wishart distributions:

``` r
burnin <- 1000
nu_mcmc <- fit$nu[-c(1:burnin), ]
colMeans(nu_mcmc)
#> [1]  8.574911 14.397351  3.310689
```

<br> True DoF:

``` r
dat$nu # true nu
#> [1]  8 12  3
```

<br> Posterior means for scale matrices of Wishart distributions:

``` r
MoE_Sigma <- Reduce("+", fit$Sigma) / length(fit$Sigma)
MoE_Sigma
#> , , 1
#> 
#>           [,1]      [,2]
#> [1,] 0.5197160 0.2103881
#> [2,] 0.2103881 0.7470847
#> 
#> , , 2
#> 
#>           [,1]      [,2]
#> [1,] 1.7637949 0.5540576
#> [2,] 0.5540576 1.3244947
#> 
#> , , 3
#> 
#>            [,1]       [,2]
#> [1,]  4.1115070 -0.1267705
#> [2,] -0.1267705  3.0385263
```

<br> Posterior means for gating coefficients:

``` r
beta_mcmc <- fit$Beta_samples[-c(1:burnin), , ]
apply(beta_mcmc, c(2, 3), mean)
#>            [,1]        [,2] [,3]
#> [1,] -0.3656861 -0.08024419    0
#> [2,] -0.9526224  2.24956385    0
#> [3,]  1.7609922  2.40287152    0
#> [4,] -0.4953755 -2.56072719    0
```

### 2. Working model: Bayesian Wishart mixture model

``` r
# fit Bayesian Wishart mixture model
set.seed(123)
fit2 <- mixturewishart(
  dat$S,
  K = 3,
  mh_sigma = c(0.2, 0.1, 0.2), # RW-MH variances
  niter = 3000, burnin = 1000
)
```

<br> Posterior means for subpopulation probabilities:

``` r
colMeans(fit2$pi[-c(1:burnin), ])
#> [1] 0.2690425 0.5088864 0.2220712
```

<br>

Posterior means for DoF of Wishart distributions:

``` r
colMeans(fit2$nu[-c(1:burnin), ])
#> [1]  7.986113 12.153338  3.284252
```

### 3. Working model: MoE-Wishart model via EM algorithm

``` r
# fit MoE-Wishart model via EM alg.
set.seed(123)
fit3 <- moewishart(
  dat$S,
  X = cbind(1, dat$X), K = 3,
  method = "em",
  niter = 3000
)
```

<br>

EM estimates for DoF of Wishart distributions:

``` r
fit3$nu
#> [1]  7.515417 13.987158  3.274665
```

<br> EM estimates for Wishart scale matrices:

``` r
fit3$Sigma
#> [[1]]
#>           [,1]      [,2]
#> [1,] 0.5591113 0.2324429
#> [2,] 0.2324429 0.8148737
#> 
#> [[2]]
#>           [,1]      [,2]
#> [1,] 1.7665723 0.5567668
#> [2,] 0.5567668 1.3336367
#> 
#> [[3]]
#>            [,1]       [,2]
#> [1,]  4.3139885 -0.1886288
#> [2,] -0.1886288  3.0983710
```

<br> EM estimates for gating coefficients:

``` r
fit3$Beta
#>             comp1      comp2 comp3
#> [1,] -0.006270492  0.1302039     0
#> [2,] -0.798302303  2.0340525     0
#> [3,]  1.598530103  2.2399293     0
#> [4,] -0.510585695 -2.3465248     0
```

### 4. Working model: Wishart mixture model via EM algorithm

``` r
# fit Wishart mixture model via EM alg.
set.seed(123)
fit4 <- mixturewishart(
  dat$S,
  K = 3,
  method = "em",
  niter = 3000
)
```

<br> EM estimates for DoF of Wishart distributions:

``` r
fit4$nu
#> [1]  2.995383  7.682819 11.309040
```

<br> EM estimate for Wishart scale matrices:

``` r
fit4$Sigma
#> [[1]]
#>           [,1]     [,2]
#> [1,]  4.048859 -1.10103
#> [2,] -1.101030  2.79641
#> 
#> [[2]]
#>           [,1]      [,2]
#> [1,] 0.5582012 0.2515063
#> [2,] 0.2515063 0.8151752
#> 
#> [[3]]
#>           [,1]      [,2]
#> [1,] 2.0930529 0.6273737
#> [2,] 0.6273737 1.5706730
```

## Reference

Mai TT, Zhao Z (2026). Mixture-of-experts Wishart model for covariance
matrices with an application to Cancer drug screening.
[arXiv:2602.13888](https://doi.org/10.48550/arXiv.2602.13888).

#' @title EM/Bayesian estimation for Wishart mixture model
#'
#' @description
#' Fit finite mixtures of Wishart-distributed SPD matrices using either a
#' Bayesian sampler or the EM algorithm. The input \code{S_list} is a list
#' of \eqn{p \times p} SPD matrices. Under component \eqn{k},
#' \eqn{S_i \mid z_i=k \sim W_p(\nu_k, \Sigma_k)} with degrees of freedom
#' \eqn{\nu_k} and SPD scale matrix \eqn{\Sigma_k}. Mixture weights
#' \eqn{\pi_k} sum to \eqn{1}.
#'
#' @name mixturewishart
#'
#' @importFrom utils combn
#' @importFrom stats kmeans
#'
#' @param S_list List of length \eqn{n} of SPD matrices, each \eqn{p \times p}.
#'   These are the observed matrices modeled by a mixture of Wisharts.
#' @param K Integer. Number of mixture components.
#' @param niter Integer. Total iterations. Bayesian mode: total MCMC
#'   iterations (including burn-in). EM mode: maximum EM iterations
#'   (alias to \code{maxiter}).
#' @param burnin Integer. Number of burn-in iterations (Bayesian mode).
#' @param method Character; one of \code{c("bayes","em")}. Selects sampler
#'   or optimizer.
#' @param thin Integer. Thinning interval for saving draws (Bayesian).
#' @param alpha Numeric vector length \eqn{K} (Dirichlet prior on
#'   \eqn{\pi}) or \code{NULL} to default to \code{rep(1, K)} (Bayesian).
#' @param nu0 Numeric. Inverse-Wishart prior df for \eqn{\Sigma_k}
#'   (Bayesian). Default: \eqn{p + 2}.
#' @param Psi0 Numeric \eqn{p \times p} SPD matrix. Inverse-Wishart prior
#'   scale for \eqn{\Sigma_k} (Bayesian). Default: \code{diag(p)}.
#' @param init_pi Optional numeric vector length \eqn{K} summing to
#'   \eqn{1}. EM initialization for mixture weights. If \code{NULL},
#'   random or data-driven initialization is used.
#' @param init_nu Optional numeric vector length \eqn{K} of initial
#'   degrees of freedom. Used in both modes.
#' @param init_Sigma Optional list of \eqn{K} SPD matrices (each
#'   \eqn{p \times p}). EM initialization for \eqn{\Sigma_k}.
#' @param marginal.z Logical. If \code{TRUE}, integrates out \eqn{\pi}
#'   when sampling \eqn{z} (collapsed step) in Bayesian mode. If
#'   \code{FALSE}, samples \eqn{z} conditional on current \eqn{\pi}.
#' @param estimate_nu Logical. If \code{TRUE}, estimate/update
#'   \eqn{\nu_k} (MH in Bayesian mode; Newton/EM in EM). If
#'   \code{FALSE}, \eqn{\nu_k} are fixed.
#' @param nu_prior_a Numeric. Prior hyperparameter \eqn{a} for
#'   \eqn{\nu_k} (Bayesian), used when \code{estimate_nu = TRUE}.
#' @param nu_prior_b Numeric. Prior hyperparameter \eqn{b} for
#'   \eqn{\nu_k} (Bayesian), used when \code{estimate_nu = TRUE}.
#' @param mh_sigma Numeric scalar or length-\eqn{K} vector. Proposal sd
#'   for MH updates on \eqn{\log(\nu_k)} (Bayesian, when estimating
#'   \eqn{\nu}).
#' @param n_restarts Integer. Number of random restarts for EM. Ignored
#'   in Bayesian mode.
#' @param restart_iters Integer. Number of short EM iterations per
#'   restart used to select a good initialization. Ignored in Bayesian
#'   mode.
#' @param tol Numeric. Convergence tolerance on absolute change of
#'   log-likelihood (EM), also used internally elsewhere.
#' @param verbose Logical. If \code{TRUE}, print progress information.
#'
#'
#' @details
#' Mixture mixture model:
#'  \eqn{p(S_i) = \sum_{k=1}^K \pi_k \, f_W(S_i \mid \nu_k, \Sigma_k)}.
#'
#' Algorithms:
#' \enumerate{
#'   \item \code{method = "bayes"}: Samples latent labels \eqn{z}, weights
#'         \eqn{\pi}, component scales \eqn{\Sigma_k}, and optionally
#'         \eqn{\nu_k}. Uses a Dirichlet prior for \eqn{\pi}, inverse-
#'         Wishart prior for \eqn{\Sigma_k}, and a prior on \eqn{\nu_k}
#'         when \code{estimate_nu = TRUE}. Degrees-of-freedom are updated
#'         via MH on \eqn{\log(\nu_k)} with proposal sd \code{mh_sigma}.
#'         Can integrate out \eqn{\pi} when sampling \eqn{z} if
#'         \code{marginal.z = TRUE}.
#'   \item \code{method = "em"}: Maximizes the observed-data log-
#'         likelihood via EM. The E-step computes responsibilities via
#'         Wishart log-densities. The M-step updates \eqn{\pi_k} and
#'         \eqn{\Sigma_k}; optionally updates \eqn{\nu_k} when
#'         \code{estimate_nu = TRUE}. Supports multiple random restarts.
#' }
#'
#' Note that
#' (i) All matrices in \code{S_list} must be SPD. Small ridge terms may be
#' added internally for stability, and
#' (ii) Multiple EM restarts are recommended for robustness on difficult datasets.
#'
#'
#' @return A list whose structure depends on \code{method}:
#' \itemize{
#'   \item For \code{method = "bayes"}:
#'     \itemize{
#'       \item \code{pi_ik}: array (\code{nsave} x \code{n} x \code{K}),
#'             saved per-observation weights.
#'       \item \code{pi}: matrix (\code{nsave} x \code{K}), saved mixture
#'             proportions.
#'       \item \code{nu}: matrix (\code{nsave} x \code{K}), saved degrees-
#'             of-freedom.
#'       \item \code{Sigma}: list of length \code{nsave}; each is an
#'             array (\eqn{p \times p \times K}) of \eqn{\Sigma_k} draws.
#'       \item \code{z}: matrix (\code{nsave} x \code{n}), saved
#'             allocations.
#'       \item \code{sigma_posterior_mean}: array (\eqn{p \times p \times
#'             K}), posterior mean of \eqn{\Sigma_k}.
#'       \item \code{loglik}: numeric vector (length \code{niter}), log-
#'             likelihood trace.
#'       \item \code{loglik_individual}: matrix (\code{niter} x
#'             \code{n}), per-observation log-likelihood.
#'     }
#'   \item For \code{method = "em"}:
#'     \itemize{
#'       \item \code{pi}: numeric vector length \eqn{K}, mixture
#'             proportions.
#'       \item \code{Sigma}: list length \code{K}, each a \eqn{p \times p}
#'             SPD matrix.
#'       \item \code{nu}: numeric vector length \code{K}, degrees-of-
#'             freedom.
#'       \item \code{tau}: matrix (\eqn{n \times K}), responsibilities.
#'       \item \code{loglik}: numeric vector, log-likelihood per EM
#'             iteration.
#'       \item \code{iterations}: integer, number of EM iterations
#'             performed.
#'     }
#' }
#'
#'
#' @examples
#'
#' # simulate data
#' set.seed(123)
#' n <- 500 # subjects
#' p <- 2
#' dat <- simData(n, p,
#'   K = 3,
#'   pis = c(0.35, 0.40, 0.25),
#'   nus = c(8, 16, 3)
#' )
#'
#' set.seed(123)
#' fit <- mixturewishart(
#'   dat$S,
#'   K = 3,
#'   mh_sigma = c(0.2, 0.1, 0.1), # tune this for MH acceptance 20-40%
#'   niter = 100, burnin = 50
#' )
#'
#' # Posterior means for degrees of freedom of Wishart distributions:
#' nu_mcmc <- fit$nu[-c(1:fit$burnin), ]
#' colMeans(nu_mcmc)
#'
#' @export
mixturewishart <- function(S_list,
                           K,
                           niter = 3000,
                           burnin = 1000,
                           method = "bayes",
                           thin = 1,
                           alpha = NULL,
                           nu0 = NULL,
                           Psi0 = NULL,
                           init_pi = NULL,
                           init_nu = NULL,
                           init_Sigma = NULL,
                           marginal.z = TRUE,
                           estimate_nu = TRUE,
                           nu_prior_a = 2,
                           nu_prior_b = 0.1,
                           mh_sigma = 1,
                           n_restarts = 3,
                           restart_iters = 20,
                           tol = 1e-6,
                           verbose = TRUE) {
  if (!method %in% c("bayes", "em")) {
    stop("Argument 'method' must be either 'bayes' or 'em'!")
  }

  # TODO: remove redundant code for the common calculations between full Bayesian and EM algorithm
  if (method == "bayes") {
    # -- 1. Pre-processing and Pre-allocation --
    n <- length(S_list)
    p <- nrow(S_list[[1]])

    if (length(mh_sigma) != K && length(mh_sigma) != 1) {
      stop("Argument 'mh_sigma' must have length 1 or K!")
    } else if (length(mh_sigma) == 1) {
      mh_sigma <- rep(mh_sigma, K)
    }

    if (!is.null(alpha)) {
      if (length(alpha) != K) {
        warning("Length of alpha (", length(alpha), ") != K (", K, "). Recycling/triming alpha to length K.")
        alpha <- rep(alpha, length.out = K)
      }
    } else {
      alpha <- rep(1, K)
    }

    # Defaults
    if (is.null(nu0)) nu0 <- p + 2
    if (is.null(Psi0)) Psi0 <- diag(p)
    if (is.null(init_nu)) init_nu <- rep(p + 2, K)

    # OPTIMIZATION: Vectorize Data
    # Flatten each p x p matrix into a row of length p^2
    # This allows fast summation and fast trace calculation
    S_mat <- t(sapply(S_list, as.vector)) # Dimension: n x (p*p)

    # OPTIMIZATION: Precompute log determinants of data
    # This part of the density never changes
    log_det_S <- sapply(S_list, function(x) determinant(x, logarithm = TRUE)$modulus)

    # Initialize Parameters
    # Use vectorized data for kmeans
    km <- kmeans(S_mat, centers = K, nstart = 5)
    z <- km$cluster

    if (is.null(init_pi)) {
      pi_k <- table(factor(z, levels = 1:K)) / n
    } else {
      if (length(init_pi) != K || sum(init_pi) != 1) {
        stop("Please specify correct 'init_pi'!")
      }
      pi_k <- init_pi
    }
    Sigma_k <- array(0, c(p, p, K))
    nu_k <- init_nu

    # Initial Sigma
    for (k in 1:K) {
      idx <- which(z == k)
      if (length(idx) == 0) {
        Sigma_k[, , k] <- Psi0 / (nu0 - p - 1)
      } else {
        # OPTIMIZATION: Fast matrix sum using colSums on vectorized data
        S_sum_vec <- colSums(S_mat[idx, , drop = FALSE])
        S_sum <- matrix(S_sum_vec, p, p)
        Sigma_k[, , k] <- (Psi0 + S_sum) / (nu0 + length(idx) * nu_k[k] - p - 1)
      }
    }

    # Storage
    ## nsave <- floor((niter - burnin) / thin)
    nsave <- floor(niter / thin)
    if (nsave < 1) nsave <- 1
    out_pi_ik <- array(NA, dim = c(nsave, n, K))
    out_pi <- matrix(NA, nrow = nsave, ncol = K)
    out_nu <- matrix(NA, nrow = nsave, ncol = K)
    out_Sigma <- vector("list", nsave)
    out_z <- matrix(NA, nrow = nsave, ncol = n)
    logliks <- numeric(niter)
    logliks_individual <- matrix(NA, nrow = niter, ncol = n)
    iter_save <- 0

    # Pre-allocate reusable vectors
    logpost <- matrix(0, n, K)
    n_k <- as.numeric(table(factor(z, levels = 1:K)))
    acc_count <- numeric(K) # record acceptance counts of MH for nu

    # Start Timer
    # start_time <- Sys.time()

    for (iter in 1:niter) {
      # --- Step 1: Update Labels z (The Heavy Lifting) ---

      for (k in 1:K) {
        # OPTIMIZATION: Invert Sigma ONLY ONCE per cluster
        Sig <- Sigma_k[, , k]

        # Cholesky is faster and more stable for determinant/inverse
        chol_Sig <- tryCatch(chol(Sig), error = function(e) chol(Sig + diag(1e-6, p)))
        log_det_Sig <- 2 * sum(log(diag(chol_Sig))) # log|Sigma|
        Sig_inv <- chol2inv(chol_Sig) # Sigma^-1

        # OPTIMIZATION: Vectorized Trace calculation
        # Tr(Sigma^-1 * S_i) is the dot product of vec(Sigma^-1) and vec(S_i)
        # We calculate this for ALL i at once via matrix multiplication
        # S_mat is (n x p^2), as.vector(Sig_inv) is (p^2 x 1) -> Result (n x 1)
        tr_val <- S_mat %*% as.vector(Sig_inv)

        # Calculate log-density for all n points
        # term1: (nu - p - 1)/2 * log|S|
        # term2: -0.5 * tr(Sig^-1 S)
        # term3: Normalizing constants involving nu and log|Sig|

        nu <- nu_k[k]

        term1 <- (nu - p - 1) / 2 * log_det_S
        term2 <- -0.5 * tr_val
        term3 <- -(nu * p / 2) * log(2) - (nu / 2) * log_det_Sig
        term4 <- -lmvgamma(nu / 2, p)

        # logpost[, k] <- log(pi_k[k] + 1e-300) + term1 + term2 + term3 + term4
        logpost[, k] <- term1 + term2 + term3 + term4
        if (marginal.z) {
          logpost[, k] <- logpost[, k] + log(alpha[k] + n_k[k] - as.numeric(z == k) + 1e-300)
        } else {
          logpost[, k] <- logpost[, k] + log(pi_k[k] + 1e-300)
        }
      }

      # Sample z
      # Vectorized sampling is hard in base R, looping sample.int is okay
      # but we can optimize the probability normalization
      # Using pure R loop for sampling is usually fast enough compared to the math above
      pi_ik <- matrix(NA, nrow = n, ncol = K)
      for (i in 1:n) {
        lp <- logpost[i, ]
        lp <- lp - max(lp)
        prob <- exp(lp)
        z[i] <- sample.int(K, 1, prob = prob)
        pi_ik[i, ] <- prob / sum(prob)
      }

      # --- Step 2: Update Weights pi ---
      n_k <- as.numeric(table(factor(z, levels = 1:K)))
      pi_k <- as.numeric(rdirichlet(1, alpha + n_k))
      # pi_k <- c(0.35, 0.40, 0.25)

      # --- Step 3: Update Sigma_k ---
      for (k in 1:K) {
        idx <- which(z == k)
        nk <- length(idx)

        if (nk == 0) {
          Sigma_k[, , k] <- sampleIW(nu0, solve(Psi0))
        } else {
          # OPTIMIZATION: Fast Sum
          S_sum_vec <- colSums(S_mat[idx, , drop = FALSE])
          S_sum <- matrix(S_sum_vec, p, p)

          nu_post <- nu0 + nk * nu_k[k]
          Psi_post <- solve(Psi0) + S_sum

          # Invert Psi_post once for sampling
          # Use tryCatch for numerical stability
          Psi_post_inv <- tryCatch(solve(Psi_post), error = function(e) solve(Psi_post + diag(1e-6, p)))

          # browser() ##TODO: check bugs in updating Sigma_k
          Sigma_k[, , k] <- sampleIW(nu_post, Psi_post_inv)
          # if (k == 1 )
          #   Sigma_k[, , k] <- matrix(c(0.5,0.2,0.2,0.7), nrow = 2)
          # if (k == 2 )
          #   Sigma_k[, , k] <- matrix(c(2,0.6,0.6,1.5), nrow = 2)
          # if (k == 3 )
          #   Sigma_k[, , k] <- matrix(c(4,0.2,0.2,3), nrow = 2)
        }
      }

      # --- Step 4: Update nu (MH) ---
      if (estimate_nu) {
        for (k in 1:K) {
          curr_nu <- nu_k[k]
          prop_log <- rnorm(1, log(curr_nu), mh_sigma[k])
          # prop_log <- log(curr_nu) + rnorm(1, 0, mh_sigma) # random-walk MH
          prop_nu <- exp(prop_log)

          if (prop_nu > p - 1 + 1e-6) {
            # We need the Likelihood sum for this cluster
            # Reuse the data stats we already know
            idx <- which(z == k)
            if (length(idx) > 0) {
              # Re-calculate only necessary parts
              # We need log|S| sum and tr(Sig^-1 S) sum

              # Grab the specific rows and sum them up for efficiency
              sum_log_det_S_k <- sum(log_det_S[idx])

              # We already have Sig_inv from Step 1?
              # No, Step 3 updated Sigma. We must re-invert current Sigma.
              Sig <- Sigma_k[, , k]
              chol_Sig <- tryCatch(chol(Sig), error = function(e) chol(Sig + diag(1e-6, p)))
              log_det_Sig <- 2 * sum(log(diag(chol_Sig)))
              # Sig_inv <- chol2inv(chol_Sig)

              # # Sum of traces = Tr(Sig^-1 * Sum(S))
              # # We can calculate Sum(S) fast
              # S_sum_vec <- colSums(S_mat[idx, , drop = FALSE])
              # sum_tr_val <- sum(as.vector(Sig_inv) * S_sum_vec)

              # Define a mini function for log-lik given Sufficient Stats
              calc_ll_nu <- function(val_nu) {
                term1 <- (val_nu - p - 1) / 2 * sum_log_det_S_k
                # term2 <- -0.5 * sum_tr_val ## this term is indep. of nu_k
                term3 <- -length(idx) * ((val_nu * p / 2) * log(2) + (val_nu / 2) * log_det_Sig)
                term4 <- -length(idx) * lmvgamma(val_nu / 2, p)
                term1 + term3 + term4
              }

              ll_old <- calc_ll_nu(curr_nu)
              ll_new <- calc_ll_nu(prop_nu)
            } else {
              ll_old <- 0
              ll_new <- 0
            }

            # Priors + Jacobian
            lp_old <- (nu_prior_a - 1) * log(curr_nu) - nu_prior_b * curr_nu + log(curr_nu)
            lp_new <- (nu_prior_a - 1) * log(prop_nu) - nu_prior_b * prop_nu + log(prop_nu)

            if (log(runif(1)) < (ll_new + lp_new) - (ll_old + lp_old)) {
              nu_k[k] <- prop_nu
              acc_count[k] <- acc_count[k] + 1
            }
          }
        }
      }
      # browser() ##TODO: check bugs in updating nu_k
      # nu_k <- c(8, 12, 3)

      # --- Calculate LogLik for history (fast approximation using Step 1 data) ---
      # We actually calculated logpost at the start of the loop (using old params).
      # We can use that for the record, or re-calc.
      # Using the one from Step 1 is "Lag-1" loglik but much faster.
      # For strictness, let's recalculate using log-sum-exp on logpost:
      # Note: logpost updated in Step 1 corresponds to Z sampling.

      # Fast LogSumExp on rows
      max_l <- apply(logpost, 1, max)
      row_sums <- exp(logpost - max_l)
      logliks[iter] <- sum(max_l + log(rowSums(row_sums)))
      logliks_individual[iter, ] <- max_l + log(rowSums(row_sums))

      # --- Save ---
      ## if (iter > burnin && ((iter - burnin) %% thin == 0)) {
      if (iter %% thin == 0) {
        iter_save <- iter_save + 1
        if (iter_save <= nsave) {
          out_pi_ik[iter_save, , ] <- pi_ik
          out_pi[iter_save, ] <- pi_k
          out_nu[iter_save, ] <- nu_k
          out_Sigma[[iter_save]] <- Sigma_k
          out_z[iter_save, ] <- z
        }
      }

      if (verbose && (iter %% 500 == 0 || iter == 1)) {
        # Calculate speed
        # elapsed <- as.numeric(difftime(Sys.time(), start_time, units = "secs"))
        # rate <- iter / elapsed
        cat(sprintf(
          "Iter %4d | LL=%.1f | acc_rate_nu=%.3f\n",
          iter, logliks[iter], acc_count / iter # min(acc_count / iter), max(acc_count / iter)
        ))
      }
    }

    sigma_posterior_mean <- Reduce("+", out_Sigma) / length(out_Sigma)

    ret <- list(
      pi_ik = out_pi_ik,
      pi = out_pi, nu = out_nu, Sigma = out_Sigma, z = out_z,
      sigma_posterior_mean = sigma_posterior_mean,
      estimate_nu = estimate_nu,
      burnin = burnin, niter = niter, thin = thin,
      loglik = logliks,
      loglik_individual = logliks_individual
    )

    class(ret) <- c("mixturewishart.bayes")
  }

  if (method == "em") {
    maxiter <- niter
    ret <- mixturewishart.em(
      S_list, K,
      n_restarts, restart_iters,
      init_pi, init_Sigma, init_nu,
      maxiter, tol, estimate_nu, verbose
    )

    ret$estimate_nu <- estimate_nu
    class(ret) <- c("mixturewishart.em")
  }

  return(ret)
}

# -- Internal function with EM algorithm for the Wishart mixture model--

mixturewishart.em <- function(S_list, K,
                              n_restarts = 3,
                              restart_iters = 20,
                              init_pi = NULL,
                              init_Sigma = NULL,
                              init_nu = NULL,
                              maxiter = 200,
                              tol = 1e-6,
                              estimate_nu = TRUE,
                              verbose = TRUE) {
  n <- length(S_list)
  p <- ncol(S_list[[1]])

  # Pre-compute log|S_i| as it is constant throughout EM
  logdetS <- sapply(S_list, function(S) {
    as.numeric(determinant(S + diag(p) * 1e-12, logarithm = TRUE)$modulus)
  })

  # --- 1. Initialization (CORRECTED) ---

  # =========================================================================
  # PHASE 1: Initialization Selection (Multiple Restarts)
  # =========================================================================

  # We only run restarts if the user did NOT provide specific starting Sigma/pi
  if (is.null(init_Sigma) && is.null(init_pi) && is.null(init_nu)) {
    if (verbose) cat("Running", n_restarts, "initialization restarts...\n")

    best_start_loglik <- -Inf
    best_params <- list()

    r <- 1
    repeat {
      ## for (r in 1:n_restarts) {

      # A. Random Initialization for this attempt
      # Default nu
      curr_nu <- if (is.null(init_nu)) rep_len(c(p + 1, p + 5), length.out = K) else init_nu
      tamtam <- rgamma(K, shape = 1)
      curr_pi <- tamtam / sum(tamtam)

      # Random centers
      rand_indices <- sample(seq_len(n), K)
      curr_Sigma <- lapply(seq_len(K), function(k) {
        idx <- rand_indices[k]
        S_list[[idx]] / curr_nu[k]
      })

      # B. Run "Small EM" (Short Loop)
      curr_loglik <- -Inf

      for (small_it in 1:restart_iters) {
        # E-Step (Simplified)
        logdens <- matrix(NA, n, K)
        for (k in seq_len(K)) {
          for (i in seq_len(n)) {
            # if(is.null(curr_Sigma[[k]])) browser()
            logdens[i, k] <- dWishart(S_list[[i]], curr_nu[k], curr_Sigma[[k]],
              logarithm = TRUE
            )
          }
          logdens[, k] <- logdens[, k] + log(curr_pi[k])
        }
        maxlog <- apply(logdens, 1, max)
        denom <- maxlog + log(rowSums(exp(logdens - maxlog)))
        logtau <- sweep(logdens, 1, denom, FUN = "-")
        tau <- exp(logtau)
        curr_loglik <- sum(denom)

        # M-Step (Simplified - update Pi and Sigma only, keep Nu fixed for stability in init)
        N_k <- colSums(tau)
        curr_pi <- N_k / n

        # Check for collapse
        if (any(N_k < 1e-6)) {
          curr_loglik <- -Inf
          break
        }

        for (k in seq_len(K)) {
          Ssum <- matrix(0, p, p)
          for (i in seq_len(n)) if (tau[i, k] > 1e-10) Ssum <- Ssum + tau[i, k] * S_list[[i]]
          curr_Sigma[[k]] <- Ssum / (N_k[k] * curr_nu[k])
        }
      }

      # C. Compare and Store
      if (verbose) cat(sprintf("  -> Restart %d: Loglik = %.2f\n", r, curr_loglik))

      if (curr_loglik > best_start_loglik) {
        best_start_loglik <- curr_loglik
        best_params <- list(pi = curr_pi, Sigma = curr_Sigma, nu = curr_nu)
      }

      if (r >= n_restarts && curr_loglik > -Inf) break
      r <- r + 1
    }

    # Set the main loop variables to the winner
    pi_k <- best_params$pi
    Sigma_k <- best_params$Sigma
    nu_k <- best_params$nu
  } else {
    if (is.null(init_Sigma) || is.null(init_pi) || is.null(init_nu)) {
      stop("Please provide all initial values of 'init_Sigma', 'init_pi' and 'init_nu'!")
    }
    # Manual initialization provided by user
    if (is.null(init_pi)) pi_k <- rep(1 / K, K) else pi_k <- init_pi
    if (is.null(init_nu)) nu_k <- rep(p + 5, K) else nu_k <- init_nu
    Sigma_k <- init_Sigma
  }

  tau <- matrix(0, n, K)
  loglik_trace <- numeric()

  for (iter in seq_len(maxiter)) {
    # --- 2. E-Step ---
    logdens <- matrix(NA, n, K)

    for (k in seq_len(K)) {
      # Vectorize this loop if possible, but loop is okay for readability
      for (i in seq_len(n)) {
        # if(is.null(Sigma_k[[k]])) browser()
        logdens[i, k] <- dWishart(S_list[[i]], nu_k[k], Sigma_k[[k]],
          logarithm = TRUE
        )
      }
      logdens[, k] <- logdens[, k] + log(pi_k[k])
    }

    # Log-Sum-Exp for numerical stability
    maxlog <- apply(logdens, 1, max)
    # Prevent underflow for very small probabilities
    denom <- maxlog + log(rowSums(exp(logdens - maxlog)))
    logtau <- sweep(logdens, 1, denom, FUN = "-")
    tau <- exp(logtau)

    current_loglik <- sum(denom)
    loglik_trace <- c(loglik_trace, current_loglik)

    if (verbose && iter %% 10 == 0) {
      cat(sprintf(
        "Iter %3d | Loglik: %.4f | Nu: %s\n",
        iter, current_loglik, paste(round(nu_k, 2), collapse = ", ")
      ))
    }

    # Convergence check
    if (iter > 2 && abs(loglik_trace[iter] - loglik_trace[iter - 1]) < tol) {
      if (verbose) cat("Converged at iteration", iter, "\n")
      break
    }

    # --- 3. M-Step ---
    N_k <- colSums(tau)
    pi_k <- N_k / n

    # Prevent collapse of clusters
    if (any(N_k < 1e-6)) {
      # Re-initialize empty cluster to a random data point
      bad_k <- which(N_k < 1e-6)
      for (bk in bad_k) {
        ridx <- sample(n, 1)
        Sigma_k[[bk]] <- S_list[[ridx]] / nu_k[bk]
        N_k[bk] <- 1 # Soft reset
      }
    }

    for (k in seq_len(K)) {
      # Weighted sum of S_i
      # Using Reduce is okay, but loop accumulation is often clearer/faster in R for lists
      Ssum <- matrix(0, p, p)
      for (i in seq_len(n)) {
        if (tau[i, k] > 1e-10) { # Sparse update optimization
          Ssum <- Ssum + tau[i, k] * S_list[[i]]
        }
      }

      # 3a. Update Sigma (conditional on current nu)
      Sigma_k[[k]] <- Ssum / (N_k[k] * nu_k[k])
    }

    for (k in seq_len(K)) {
      # 3b. Update Nu (if requested)
      if (estimate_nu) {
        # Profile Log-Likelihood maximization for nu
        # Target: log(nu/2) + psi(nu/2 + ...) terms vs Data Stats

        T1k <- sum(tau[, k] * logdetS)
        logdetSig <- as.numeric(determinant(Sigma_k[[k]] + diag(p) * 1e-12,
          logarithm = TRUE
        )$modulus)

        lhs <- (T1k / N_k[k]) - logdetSig - p * log(2)

        # Newton-Raphson functions

        f_optim <- function(v) {
          # We want to find root of: LHS - sum(digamma(v/2 + ...))
          # , strictly: E[log|W|] = log|Sigma| + p*log(2) + sum(digamma(...))
          # So sum(digamma(...)) = E[log|W|] - log|Sigma| - p*log(2) = LHS.
          # So we want: LHS - sum(digamma(...)) = 0
          lhs - sum(digamma(v / 2 + (1 - seq_len(p)) / 2))
        }

        fp_optim <- function(v) {
          -0.5 * sum(trigamma(v / 2 + (1 - seq_len(p)) / 2))
        }

        # Newton Iteration
        curr_nu <- nu_k[k]
        for (nm in 1:50) {
          val <- f_optim(curr_nu)
          grad <- fp_optim(curr_nu)
          if (is.na(val) || is.na(grad) || abs(grad) < 1e-8) break

          step <- val / grad
          curr_nu <- curr_nu - step

          # Constraints
          if (curr_nu <= p) curr_nu <- p + 0.1
          if (curr_nu > 1e3) curr_nu <- 1e3
          if (abs(step) < 1e-4) break
        }
        nu_k[k] <- curr_nu

        # 3c. Consistency update: Re-calc Sigma with new nu
        # Sigma_k[[k]] <- Ssum / (N_k[k] * nu_k[k])
      }
    }
    # if (any(nu_k > 1e3)) break
  }

  list(
    pi = pi_k, Sigma = Sigma_k, nu = nu_k, tau = tau,
    loglik = loglik_trace, iterations = iter
  )
}

#' @title density of Wishart distribution
#'
#' @description
#' Compute the (log) density of a \eqn{p}-dimensional Wishart distribution
#' \eqn{W_p(\nu, \Sigma)} at an SPD matrix \eqn{S}. Returns either the
#' log-density or the density depending on \code{logarithm}.
#'
#' @name dWishart
#'
#' @importFrom stats rbinom rnorm runif rexp rgamma
#'
#' @param S Numeric \eqn{p \times p} SPD matrix at which to evaluate the
#'   density.
#' @param nu Numeric. Degrees of freedom \eqn{\nu} (must exceed \eqn{p-1}).
#' @param Sigma Numeric \eqn{p \times p} SPD scale matrix \eqn{\Sigma}.
#' @param detS_val Optional numeric. Precomputed \eqn{\log|S|} to reuse;
#'   if \code{NULL}, it is computed internally.
#' @param logarithm Logical. If \code{TRUE}, return log-density;
#'   otherwise return density.
#'
#'
#' @details
#' Let \eqn{S \sim W_p(\nu, \Sigma)} with degrees of freedom \eqn{\nu}
#' and scale matrix \eqn{\Sigma} (SPD). The density is:
#' \deqn{
#'   f(S \mid \nu, \Sigma) =
#'   \frac{|S|^{(\nu - p - 1)/2}\,
#'         \exp\{-\tfrac{1}{2}\,\mathrm{tr}(\Sigma^{-1}S)\}}
#'        {2^{\nu p/2}\,|\Sigma|^{\nu/2}\,\Gamma_p(\nu/2)},
#' }
#' where \eqn{\Gamma_p(\cdot)} is the multivariate gamma function and
#' \eqn{p} is the dimension.
#'
#' Note that
#' (i) \code{detS_val} can be supplied to avoid recomputing \eqn{\log|S|},
#'  which is useful inside EM/MCMC loops, and
#'  (ii) small diagonal jitter is added internally to \eqn{S} and \eqn{\Sigma}
#'  when computing determinants or solves for numerical stability.
#'
#' Constraints: (i) \eqn{S} and \eqn{\Sigma} must be SPD, and (ii) the Wishart
#' requires \eqn{\nu > p - 1}.
#'
#'
#' @return A numeric scalar: the log-density if \code{logarithm = TRUE},
#'   otherwise the density.
#'
#' @examples
#'
#' set.seed(123)
#' p <- 3
#' # Construct an SPD Sigma
#' A <- matrix(rnorm(p * p), p, p)
#' Sigma <- crossprod(A) + diag(p) * 0.5
#' # Draw a Wishart matrix using base stats::rWishart()
#' W <- drop(rWishart(1, df = p + 5, Sigma = Sigma))
#' # Evaluate log-density at W
#' dWishart(W, nu = p + 5, Sigma = Sigma, logarithm = TRUE)
#'
#' @export
dWishart <- function(S, nu, Sigma, detS_val = NULL, logarithm = TRUE) {
  # log density of Wishart S ~ W_p(nu, Sigma)

  p <- ncol(S)

  # Calculate log-determinants safely
  if (is.null(detS_val)) {
    detS_val <- as.numeric(determinant(S + diag(p) * 1e-6, logarithm = TRUE)$modulus)
  }
  detSig_res <- determinant(Sigma + diag(p) * 1e-6, logarithm = TRUE)
  ldSigma <- as.numeric(detSig_res$modulus)

  # Calculate trace(Sigma^{-1} S) efficiently
  # solving Sigma x = S is better than explicit inverse
  invSigma_S <- sum(diag(solve(Sigma + diag(1e-6, p), S)))

  term <- ((nu - p - 1) / 2) * detS_val - (nu / 2) * ldSigma - 0.5 * invSigma_S
  ret <- term - (nu * p / 2) * log(2) - lmvgamma(nu / 2, p)

  if (!logarithm) {
    ret <- exp(ret)
  }

  return(ret)
}

#' @title Trace plot of parameters
#'
#' @description
#' Draw MCMC trace plot of parameters
#'
#' @name plotMCMC
#'
#' @importFrom graphics legend matplot
#'
#' @param object returned object from Bayesian MoE or mixture model
#' @param estimator specified parameter for traceplot
#' @param coeff.idx choice of one covariate to show its coefficients in clusters
#'
#'
#' @return A trace plot 
#'
#'
#' @examples
#'
#' # simulate data
#' set.seed(123)
#' n <- 500 # subjects
#' p <- 2
#' # True gating coefficients (last column zero)
#' set.seed(123)
#' Xq <- 3
#' K <- 3
#' betas <- matrix(runif(Xq * K, -2, 2), nrow = Xq, ncol = K)
#' betas[, K] <- 0
#' dat <- simData(n, p,
#'   Xq = 3, K = 3, betas = betas,
#'   pis = c(0.35, 0.40, 0.25),
#'   nus = c(8, 16, 3)
#' )
#'
#' set.seed(123)
#' fit <- moewishart(
#'   dat$S,
#'   X = cbind(1, dat$X), K = 3,
#'   mh_sigma = c(0.2, 0.1, 0.1), # RW-MH variances (length K)
#'   mh_beta = c(0.2, 0.2), # RW-MH variances (length K-1)
#'   niter = 500, burnin = 200
#' )
#' 
#' plotMCMC(fit, estimator = "nu")
#'
#' @export
plotMCMC <- function(object, estimator = "nu", coeff.idx = 2) {
  
  # check if the model fitted object was from a Bayesian method
  if (inherits(object, "moewishart.bayes") || 
      inherits(object, "mixturewishart.bayes")) {
    
    if (inherits(object, "moewishart.bayes")) {
      if (!estimator %in% c("nu", "sigma", "beta", "loglik")) {
        stop("The argument 'estimator' must be one of c('nu', 'sigma', 'beta', 'loglik')!")
      }
    } else {
      if (!estimator %in% c("nu", "sigma", "pi", "loglik")) {
        stop("The argument 'estimator' must be one of c('nu', 'sigma', 'pi', 'loglik')!")
      }
    } 
  } else {
    stop("The model fitted object must be Bayesian MoE or Bayesian mixture model.")
  }
  
  # number of clusters/components
  K <- NCOL(object$nu)
  
  # traceplot for nu
  if (estimator == "nu") {
    mat <- object$nu
    matplot(mat, type = "l", lty = 1, col = 1:K, 
            xlab = "MCMC iteration", ylab = "Value",
            main = "Trace plot")
    
    legend_labels <- as.expression(
      lapply(1:K, function(i) bquote(nu[.(i)]))
    )
    legend("topright", legend = legend_labels, col = 1:K, lty = 1)
  }
  
  # traceplot for Sigma
  if (estimator == "sigma") {
    Sigma <- sapply(1:length(object$Sigma), function(xx){
      s0 <- object$Sigma[[xx]]
      sapply(1:K, function(k) { 
        as.numeric(determinant(s0[,,k], logarithm = TRUE)$modulus)
      })
    })
    mat <- t(Sigma)
    matplot(mat, type = "l", lty = 1, col = 1:K, 
            xlab = "MCMC iteration", ylab = "Value",
            main = "Trace plot")
    
    legend_labels <- as.expression(
      lapply(1:K, function(i) bquote(log~abs(Sigma[.(i)])))
    )
    legend("topright", legend = legend_labels, col = 1:K, lty = 1)
  }
  
  # traceplot for betas in MoE model
  if (estimator == "beta") {
    betas <- object$Beta_samples[, , -K] # remove (last) reference group
    coeff.idx <- 2
    mat <- betas[, coeff.idx, ] # only keep the 2nd coeff (first covariate's effect if including intercept)
    
    matplot(mat, type = "l", lty = 1, col = 1:(K-1), 
            xlab = "MCMC iteration", ylab = "Value",
            main = paste0("Trace plot of X", coeff.idx, "'s effects in various clusters"))
    
    legend_labels <- as.expression(
      lapply(1:(K-1), function(i) bquote(beta[2~.(i)]))
    )
    legend("topright", legend = legend_labels, col = 1:(K-1), lty = 1)
  }
  
  # traceplot for pi in mixture model
  if (estimator == "pi") {
    mat <- object$pi
    matplot(mat, type = "l", lty = 1, col = 1:K, 
            xlab = "MCMC iteration", ylab = "Value",
            main = "Trace plot")
    
    legend_labels <- as.expression(
      lapply(1:K, function(i) bquote(pi[.(i)]))
    )
    legend("topright", legend = legend_labels, col = 1:K, lty = 1)
  }
  
  # traceplot for log-likelihoods
  if (estimator == "loglik") {
    plot(object$loglik, type = "l", lty = 1, 
         xlab = "MCMC iteration", ylab = "Value",
         main = paste0("Trace plot of log-likelihoods"))
  }
  
}

#' @title Log multivariate gamma function
#'
#' @description
#' Compute the log of the multivariate gamma function \eqn{\log \Gamma_p(a)}
#' for dimension \eqn{p} and parameter \eqn{a}.
#'
#' @name lmvgamma
#'
#' @importFrom stats rbinom rnorm runif rexp rgamma
#'
#' @param a Numeric. Argument of \eqn{\Gamma_p(\cdot)} (often \eqn{\nu/2}
#'   in Wishart contexts).
#' @param p Integer. Dimension \eqn{p} of the multivariate gamma.
#'
#'
#' @details
#' The multivariate gamma function \eqn{\Gamma_p(a)} is defined by:
#' \deqn{
#'   \Gamma_p(a)
#'   = \pi^{\,p(p-1)/4} \prod_{j=1}^{p} \Gamma\!\left(a + \frac{1-j}{2}\right).
#' }
#'
#' Constraints: (i) \eqn{p \in \{1,2,\dots\}} (positive integer), and
#'   (ii) \eqn{a > (p-1)/2} to keep all gamma terms finite (as in the Wishart
#'   normalization constant).
#'
#'
#' @return A numeric scalar equal to \eqn{\log \Gamma_p(a)}.
#'
#'
#' @examples
#'
#' # Dimension
#' p <- 3
#' # Evaluate log multivariate gamma at a = nu/2
#' nu <- p + 5
#' lmvgamma(a = nu / 2, p = p)
#'
#' @export
lmvgamma <- function(a, p) {
  p * (p - 1) / 4 * log(pi) + sum(lgamma(a + (1 - 1:p) / 2))
}

#' @title Fast sampler for the inverse-Wishart distribution
#'
#' @description
#' Draw a random sample from an inverse-Wishart distribution
#' \eqn{\mathcal{IW}_p(\nu, \Psi)} using the identity
#' \eqn{S \sim \mathcal{IW}_p(\nu, \Psi)} iff
#' \eqn{S^{-1} \sim W_p(\nu, \Psi^{-1})}. This implementation accepts
#' \eqn{\Psi^{-1}} directly for speed.
#'
#' @name sampleIW
#'
#' @importFrom stats rbinom rnorm runif rexp rgamma pnorm rWishart toeplitz
#'
#' @param nu Numeric. Degrees of freedom \eqn{\nu} of the inverse-Wishart
#'   (must exceed \eqn{p - 1}).
#' @param Psi_inv Numeric \eqn{p \times p} SPD matrix equal to
#'   \eqn{\Psi^{-1}}, the inverse of the inverse-Wishart scale matrix.
#'
#'
#' @details
#' Sampling scheme:
#' \itemize{
#'   \item Sample \eqn{W \sim W_p(\nu, \Psi^{-1})} using \code{rWishart}.
#'   \item Return \eqn{S = W^{-1}}, which has
#'         \eqn{\mathcal{IW}_p(\nu, \Psi)}.
#' }
#'
#' Parameterization:
#' \itemize{
#'   \item \eqn{\nu} is the degrees of freedom, must satisfy
#'         \eqn{\nu > p - 1}.
#'   \item \eqn{\Psi} is the SPD scale matrix of the inverse-Wishart.
#'         This function expects its inverse \eqn{\Psi^{-1}} as input
#'         for efficiency (avoids repeated matrix inversions).
#' }
#'
#' Note that:
#' (i) internally calls \code{rWishart(1, df = nu, Sigma = Psi_inv)}, and
#' (ii) returns \code{solve(W)}; if numerical issues arise, consider
#'         adding a small ridge to \eqn{\Psi^{-1}} prior to sampling.
#'
#'
#' @return A \eqn{p \times p} SPD matrix \eqn{S} distributed as
#'   \eqn{\mathcal{IW}_p(\nu, \Psi)}.
#'
#' @examples
#'
#' set.seed(123)
#' p <- 3
#' # Construct an SPD scale matrix Psi
#' A <- matrix(rnorm(p * p), p, p)
#' Psi <- crossprod(A) + diag(p) * 0.5
#' Psi_inv <- solve(Psi)
#'
#' # Draw one inverse-Wishart sample
#' S <- sampleIW(nu = p + 5, Psi_inv = Psi_inv)
#' S
#'
#' @export
sampleIW <- function(nu, Psi_inv) {
  # Psi_inv is already the inverse of the scale matrix (or solved)
  # To sample Inv-Wishart(nu, Psi): sample W ~ Wishart(nu, solve(Psi))
  # Here we assume Psi_inv = solve(Psi) is passed for speed
  W <- rWishart(1, df = nu, Sigma = Psi_inv)[, , 1]
  solve(W)
}

#' @title EM/Bayesian estimation for Wishart MoE model
#'
#' @description
#' Fit a mixture-of-experts model for symmetric positive-definite (SPD)
#' matrices with covariate-dependent mixing proportions (gating network).
#' Components are Wishart-distributed. Supports Bayesian sampling and
#' EM-based maximum-likelihood estimation.
#'
#' @name moewishart
#'
#' @importFrom utils combn
#' @importFrom stats optim
#'
#' @param S_list List of length \eqn{n} of SPD matrices, each \eqn{p \times p}.
#'   These are the observed responses modeled by the MoE.
#' @param X Numeric matrix \eqn{n \times q} of covariates for the gating
#'   network. Include an intercept column if desired.
#' @param K Integer. Number of mixture components (experts).
#' @param niter Integer. Total iterations. Bayesian mode: total MCMC
#'   iterations (including burn-in). EM mode: maximum EM iterations.
#' @param burnin Integer. Number of burn-in iterations (Bayesian mode).
#' @param method Character; one of \code{c("bayes", "em")}. Selects
#'   sampler or optimizer.
#' @param thin Integer. Thinning interval for saving draws (Bayesian).
#' @param nu0 Numeric. Inverse-Wishart prior df for \eqn{\Sigma_k}
#'   (Bayesian). Default: \eqn{p + 2} if \code{NULL}.
#' @param Psi0 Numeric \eqn{p \times p} SPD matrix. Inverse-Wishart prior
#'   scale for \eqn{\Sigma_k} (Bayesian). Default: \code{diag(p)} if
#'   \code{NULL}.
#' @param init_nu Optional numeric vector length \eqn{K} of initial dfs
#'   \eqn{\nu_k}. Used for initialization.
#' @param estimate_nu Logical. If \code{TRUE}, estimate \eqn{\nu_k}
#'   (MH in Bayesian; Newton/EM in EM). If \code{FALSE}, keep \eqn{\nu_k}
#'   fixed at \code{init_nu}.
#' @param nu_prior_a Numeric. Prior hyperparameter \eqn{a} for \eqn{\nu_k}
#'   (Bayesian), used when \code{estimate_nu = TRUE}.
#' @param nu_prior_b Numeric. Prior hyperparameter \eqn{b} for \eqn{\nu_k}
#'   (Bayesian), used when \code{estimate_nu = TRUE}.
#' @param mh_sigma Numeric scalar or length-\eqn{K} vector. Proposal sd
#'   for MH updates on \eqn{\log(\nu_k)} (Bayesian, when estimating
#'   \eqn{\nu}).
#' @param mh_beta Numeric scalar or length-\eqn{K-1} vector. Proposal sd
#'   for MH updates of the free \eqn{B} columns (Bayesian).
#' @param sigma_beta Numeric. Prior sd of the Gaussian prior on \eqn{B}
#'   (Bayesian).
#' @param init Optional list with fields for EM initialization, e.g.,
#'   \code{beta}, \code{Sigma}, \code{nu}. See return structure.
#' @param tol Numeric. Convergence tolerance on absolute change of
#'   log-likelihood (EM), also used internally.
#' @param ridge Numeric. Small diagonal ridge added to \eqn{\Sigma_k}
#'   updates in EM for numerical stability.
#' @param verbose Logical. If \code{TRUE}, print progress information.
#'
#'
#' @details
#' MoE-Wishart Model:
#' \itemize{
#'   \item Observation: \eqn{S_i} is a \eqn{p \times p} SPD matrix. Given
#'         allocation \eqn{z_i=k}, \eqn{S_i \mid z_i \sim W_p(\nu_k,
#'         \Sigma_k)} with df \eqn{\nu_k} and scale \eqn{\Sigma_k}.
#'   \item Gating (MoE): Let \eqn{X_i} be \eqn{q}-dimensional covariates.
#'         Mixing weights \eqn{\pi_{ik} = \Pr(z_i=k \mid X_i)} follow a
#'         softmax regression:
#'         \eqn{\pi_{ik} = \exp(\eta_{ik})/\sum_{j=1}^K \exp(\eta_{ij})},
#'         where \eqn{\eta_i = X_i^\top B}, \eqn{B} is
#'         \eqn{q \times K}. Identifiability: last column of \eqn{B}
#'         is fixed to zero.
#' }
#'
#' Algorithms:
#' \enumerate{
#'   \item Bayesian (\code{method = "bayes"}): Metropolis-within-Gibbs
#'         sampler for \eqn{z}, \eqn{\Sigma_k}, optional \eqn{\nu_k}, and
#'         \eqn{B}. Gaussian priors on \eqn{B} with sd
#'         \code{sigma_beta}. Proposals use \code{mh_sigma} for
#'         \eqn{\log(\nu_k)} and \code{mh_beta} for \eqn{B}.
#'   \item EM (\code{method = "em"}): E-step responsibilities using
#'         Wishart log-densities and softmax gating. M-step updates
#'         \eqn{\Sigma_k}, optional \eqn{\nu_k}, and \eqn{B} via
#'         weighted multinomial logistic regression (BFGS).
#' }
#'
#' Note that:
#' (i) include an intercept column in \code{X}; none is added by default, and
#' (ii) all \code{S_list} elements must be SPD. A small \code{ridge} may be
#' added for stability.
#'
#'
#' @return A list whose fields depend on \code{method}:
#' \itemize{
#'   \item For \code{method = "bayes"}:
#'     \itemize{
#'       \item \code{Beta_samples}: array (\code{nsave} x \code{q} x
#'             \code{K}), saved draws of \eqn{B} (last column zero).
#'       \item \code{nu}: matrix (\code{nsave} x \code{K}), draws
#'             of \eqn{\nu_k}.
#'       \item \code{Sigma}: list of length \code{nsave}; each
#'             element is an array (\eqn{p \times p \times K}) of
#'             \eqn{\Sigma_k} draws.
#'       \item \code{z_samples}: matrix (\code{nsave} x \code{n}), draws
#'             of allocations.
#'       \item \code{pi_ik}: array (\code{nsave} x \code{n} x \code{K}),
#'             per-observation gating probabilities.
#'       \item \code{pi_mean}: matrix (\code{n} x \code{K}), posterior
#'             mean of gating probabilities.
#'       \item \code{loglik}: numeric vector (length \code{niter}),
#'             log-likelihood trace.
#'       \item \code{loglik_individual}: matrix (\code{niter} x
#'             \code{n}), per-observation log-likelihood.
#'     }
#'   \item For \code{method = "em"}:
#'     \itemize{
#'       \item \code{K, p, q, n}: problem dimensions.
#'       \item \code{Beta}: matrix (\eqn{q \times K}), gating coefficients
#'             with last column zero (reference class).
#'       \item \code{Sigma}: list length \code{K}, each a \eqn{p \times p}
#'             SPD matrix (scale).
#'       \item \code{nu}: numeric vector length \code{K}, degrees of
#'             freedom.
#'       \item \code{gamma}: matrix (\eqn{n \times K}), final
#'             responsibilities.
#'       \item \code{loglik}: numeric vector, log-likelihood by EM
#'             iteration.
#'       \item \code{iter}: integer, number of EM iterations performed.
#'     }
#' }
#'
#'
#' @examples
#'
#' # simulate data
#' set.seed(123)
#' n <- 500 # subjects
#' p <- 2
#' # True gating coefficients (last column zero)
#' set.seed(123)
#' Xq <- 3
#' K <- 3
#' betas <- matrix(runif(Xq * K, -2, 2), nrow = Xq, ncol = K)
#' betas[, K] <- 0
#' dat <- simData(n, p,
#'   Xq = 3, K = 3, betas = betas,
#'   pis = c(0.35, 0.40, 0.25),
#'   nus = c(8, 16, 3)
#' )
#'
#' set.seed(123)
#' fit <- moewishart(
#'   dat$S,
#'   X = cbind(1, dat$X), K = 3,
#'   mh_sigma = c(0.2, 0.1, 0.1), # RW-MH variances (length K)
#'   mh_beta = c(0.2, 0.2), # RW-MH variances (length K-1)
#'   niter = 500, burnin = 200
#' )
#'
#' # Posterior means for degrees of freedom of Wishart distributions:
#' nu_mcmc <- fit$nu[-c(1:fit$burnin), ]
#' colMeans(nu_mcmc)
#'
#' @export
moewishart <- function(S_list,
                       X, # n x q matrix of covariates for gating
                       K,
                       niter = 3000,
                       burnin = 1000,
                       method = "bayes",
                       thin = 1,
                       nu0 = NULL,
                       Psi0 = NULL,
                       init_nu = NULL,
                       estimate_nu = TRUE,
                       nu_prior_a = 2, nu_prior_b = 0.1,
                       mh_sigma = 0.1,
                       mh_beta = 0.05, # MH proposal sd for gating coeffs
                       sigma_beta = 10, # Gaussian prior sd for beta
                       init = NULL,
                       tol = 1e-6,
                       ridge = 1e-8,
                       verbose = TRUE) {
  # Mixture-of-Experts Gibbs sampler for Wishart clusters
  #   S_list: list of n SPD matrices (p x p)
  #   X      : n x q covariate matrix for gating network (include intercept if desired)
  #   K      : number of experts/clusters
  #
  # Returns samples for Sigma_k, nu_k, z, and gating coefficients Beta


  if (!method %in% c("bayes", "em")) {
    stop("Argument 'method' must be either 'bayes' or 'em'!")
  }

  # TODO: remove redundant code for the common calculations between full Bayesian and EM algorithm
  if (method == "bayes") {
    n <- length(S_list)
    p <- nrow(S_list[[1]])
    q <- ncol(X)
    if (n != nrow(X)) stop("Number of rows in X must equal length(S_list).")

    if (length(mh_sigma) != K && length(mh_sigma) != 1) {
      stop("Argument 'mh_sigma' must have length 1 or K!")
    } else if (length(mh_sigma) == 1) {
      mh_sigma <- rep(mh_sigma, K)
    }
    if (length(mh_beta) != K - 1 && length(mh_beta) != 1) {
      stop("Argument 'mh_beta' must have length 1 or K-1!")
    } else if (length(mh_beta) == 1) {
      mh_beta <- rep(mh_beta, K - 1)
    }

    # Priors / defaults
    if (is.null(nu0)) nu0 <- p + 2
    if (is.null(Psi0)) Psi0 <- diag(p)
    if (is.null(init_nu)) init_nu <- rep(p + 2, K)

    # Vectorize S for fast trace computations
    S_mat <- t(sapply(S_list, as.vector)) # n x p^2
    log_det_S <- sapply(S_list, function(x) as.numeric(determinant(x, logarithm = TRUE)$modulus))

    # initialize z (kmeans on vectorized matrices)
    km <- kmeans(S_mat, centers = K, nstart = 5)
    z <- km$cluster

    # initialize cluster params
    Sigma_k <- array(0, c(p, p, K))
    nu_k <- init_nu
    for (k in 1:K) {
      idx <- which(z == k)
      if (length(idx) == 0) {
        Sigma_k[, , k] <- Psi0 / (nu0 - p - 1 + 1e-8)
      } else {
        S_sum_vec <- colSums(S_mat[idx, , drop = FALSE])
        S_sum <- matrix(S_sum_vec, p, p)
        Sigma_k[, , k] <- (Psi0 + S_sum) / (nu0 + length(idx) * nu_k[k] - p - 1)
      }
    }

    # Initialize gating coefficients Beta: q x (K-1), last column zero for identifiability
    Beta <- matrix(0, nrow = q, ncol = K) # we keep full K but enforce Beta[,K] = 0
    Beta[, 1:(K - 1)] <- matrix(rnorm(q * (K - 1), 0, 0.1), nrow = q, ncol = K - 1)
    Beta[, K] <- 0

    pi_ik <- compute_pi_ik(X, Beta) # n x K

    # Storage
    ## nsave <- floor((niter - burnin) / thin)
    nsave <- floor(niter / thin)
    if (nsave < 1) nsave <- 1
    out_beta <- array(NA, dim = c(nsave, q, K)) # store Beta at saves
    out_nu <- matrix(NA, nrow = nsave, ncol = K)
    out_Sigma <- vector("list", nsave)
    out_z <- matrix(NA, nrow = nsave, ncol = n)
    out_pi_ik <- array(NA, dim = c(nsave, n, K))
    out_pi_mean <- array(0, dim = c(n, K)) # accumulate posterior mean of pi_ik
    logliks <- numeric(niter)
    logliks_individual <- matrix(NA, nrow = niter, ncol = n)
    iter_save <- 0

    # pre-alloc
    logpost <- matrix(0, n, K)
    acc_count_nu <- numeric(K) # record acceptance counts of MH for nu
    acc_count_beta <- c(numeric(K - 1), NA) # record acceptance counts of MH for beta
    free_idx_cols <- 1:(K - 1)

    # Spike-and-slab indicators for Beta (q x (K-1))
    # pi_gamma <- 0.5
    # Gamma <- matrix(rbinom(q*(K-1), 1, pi_gamma), nrow = q, ncol = K-1)
    # Gama_samples =  array(NA, dim = c(nsave, q, K-1))

    # start_time <- Sys.time()
    for (iter in 1:niter) {
      # --- Step 1: compute log-likelihood parts per cluster (vectorized) ---
      for (k in 1:K) {
        Sig <- Sigma_k[, , k]
        chol_Sig <- tryCatch(chol(Sig), error = function(e) chol(Sig + diag(1e-8, p)))
        log_det_Sig <- 2 * sum(log(diag(chol_Sig)))
        Sig_inv <- chol2inv(chol_Sig)
        tr_val <- S_mat %*% as.vector(Sig_inv) # n x 1
        nu <- nu_k[k]
        term1 <- (nu - p - 1) / 2 * log_det_S
        term2 <- -0.5 * tr_val
        term3 <- -(nu * p / 2) * log(2) - (nu / 2) * log_det_Sig
        term4 <- -lmvgamma(nu / 2, p)
        # log posterior (pointwise) = log pi_ik + likelihood terms
        logpost[, k] <- log(pi_ik[, k] + 1e-300) + term1 + term2 + term3 + term4
      }

      # --- Step 2: sample z (categorical per i using pi_ik and likelihood) ---
      for (i in 1:n) {
        lp <- logpost[i, ]
        lp <- lp - max(lp)
        prob <- exp(lp)
        prob <- prob / sum(prob)
        z[i] <- sample.int(K, 1, prob = prob)
      }

      # --- Step 3: update gating coefficients Beta via MH ---
      # We'll update the (K-1) free columns jointly (size q*(K-1)).
      # Flatten current free parameters

      # free_idx_cols <- 1:(K - 1)

      # Update Beta columnwise
      for (free_idx_cols in 1:(K - 1)) {
        Beta_free <- as.vector(Beta[, free_idx_cols]) # length q*(K-1)
        prop <- Beta_free + rnorm(length(Beta_free), 0, mh_beta[free_idx_cols])
        Beta_prop <- Beta
        Beta_prop[, free_idx_cols] <- prop # matrix(prop, nrow = q, ncol = K - 1)
        # Beta_prop[, K] <- 0
        # compute new pi_ik (n x K)
        pi_prop <- compute_pi_ik(X, Beta_prop)
        # log prior for Beta (Gaussian iid)
        lp_prior_old <- -0.5 * sum(Beta^2) / (sigma_beta^2)
        lp_prior_new <- -0.5 * sum(Beta_prop^2) / (sigma_beta^2)
        # log-likelihood of labels given gating: sum_i log pi_i,z[i] (only depends on z)
        ll_old <- sum(log(pi_ik[cbind(1:n, z)] + 1e-300))
        ll_new <- sum(log(pi_prop[cbind(1:n, z)] + 1e-300))
        log_accept <- (ll_new + lp_prior_new) - (ll_old + lp_prior_old)
        if (log(runif(1)) < log_accept) {
          Beta <- Beta_prop
          pi_ik <- pi_prop
          # optionally track acceptance
          acc_count_beta[free_idx_cols] <- acc_count_beta[free_idx_cols] + 1
        }
      }

      # Identifiability constraint
      Beta[, K] <- 0


      # # --- Step 3b: update Gamma indicators (Gibbs) ---
      # # use exact posterior p(gamma=1 | beta) \propto pi_gamma * N(beta | 0, sigma_beta^2)
      # beta_cur <- Beta[, free_idx_cols]  # q x (K-1)
      # for (j in 1:q) {
      #   for (k in 1:(K-1)) {
      #     bval <- beta_cur[j, k]
      #     # log-prob (unnormalized) for gamma = 1 and gamma = 0
      #     log_p1 <- log(pi_gamma + 1e-300) - 0.5 * (bval^2) / (sigma_beta^2) - 0.5 * log(2 * pi * sigma_beta^2)
      #     log_p0 <- log(1 - pi_gamma + 1e-300) - 0.5 * (bval^2) / (tau0^2) - 0.5 * log(2 * pi * tau0^2)
      #     # numeric stable conversion to probability
      #     maxlog <- max(log_p1, log_p0)
      #     p1 <- exp(log_p1 - maxlog)
      #     p0 <- exp(log_p0 - maxlog)
      #     prob1 <- p1 / (p1 + p0)
      #     Gamma[j, k] <- rbinom(1, 1, prob1)
      #   }
      # }

      # --- Step 4: update Sigma_k using current assignments z ---
      for (k in 1:K) {
        idx <- which(z == k)
        nk <- length(idx)
        if (nk == 0) {
          # sample from prior-ish fallback
          Sigma_k[, , k] <- Psi0 / (nu0 - p - 1 + 1e-8)
        } else {
          S_sum_vec <- colSums(S_mat[idx, , drop = FALSE])
          S_sum <- matrix(S_sum_vec, p, p)
          nu_post <- nu0 + nk * nu_k[k]
          Psi_post <- Psi0 + S_sum
          Psi_post_inv <- tryCatch(solve(Psi_post), error = function(e) solve(Psi_post + diag(1e-8, p)))
          Sigma_k[, , k] <- sampleIW(nu_post, Psi_post_inv)
        }
      }

      # --- Step 5: update nu_k (MH) as before ---
      if (estimate_nu) {
        for (k in 1:K) {
          curr_nu <- nu_k[k]
          prop_log <- rnorm(1, log(curr_nu), mh_sigma[k])
          prop_nu <- exp(prop_log)
          if (prop_nu > p - 1 + 1e-8) {
            idx <- which(z == k)
            if (length(idx) > 0) {
              sum_log_det_S_k <- sum(log_det_S[idx])
              Sig <- Sigma_k[, , k]
              chol_Sig <- tryCatch(chol(Sig), error = function(e) chol(Sig + diag(1e-8, p)))
              log_det_Sig <- 2 * sum(log(diag(chol_Sig)))
              # Sig_inv <- chol2inv(chol_Sig)
              # S_sum_vec <- colSums(S_mat[idx, , drop = FALSE])
              # sum_tr_val <- sum(as.vector(Sig_inv) * S_sum_vec)
              calc_ll_nu <- function(val_nu) {
                term1 <- (val_nu - p - 1) / 2 * sum_log_det_S_k
                # term2 <- -0.5 * sum_tr_val ## this term is indep. of nu_k
                term3 <- -length(idx) * ((val_nu * p / 2) * log(2) + (val_nu / 2) * log_det_Sig)
                term4 <- -length(idx) * lmvgamma(val_nu / 2, p)
                term1 + term3 + term4
              }
              ll_old <- calc_ll_nu(curr_nu)
              ll_new <- calc_ll_nu(prop_nu)
            } else {
              ll_old <- 0
              ll_new <- 0
            }
            lp_old <- (nu_prior_a - 1) * log(curr_nu) - nu_prior_b * curr_nu + log(curr_nu)
            lp_new <- (nu_prior_a - 1) * log(prop_nu) - nu_prior_b * prop_nu + log(prop_nu)
            if (log(runif(1)) < (ll_new + lp_new) - (ll_old + lp_old)) {
              nu_k[k] <- prop_nu
              acc_count_nu[k] <- acc_count_nu[k] + 1
            }
          }
        }
      }

      # --- Compute loglik approx for monitoring ---
      max_l <- apply(logpost, 1, max)
      row_sums <- exp(logpost - max_l)
      logliks[iter] <- sum(max_l + log(rowSums(row_sums)))
      logliks_individual[iter, ] <- max_l + log(rowSums(row_sums))

      # --- Save samples after burnin and thinning ---
      ## if (iter > burnin && ((iter - burnin) %% thin == 0)) {
      if (iter %% thin == 0) {
        iter_save <- iter_save + 1
        if (iter_save <= nsave) {
          out_beta[iter_save, , ] <- Beta
          out_nu[iter_save, ] <- nu_k
          out_Sigma[[iter_save]] <- Sigma_k
          out_z[iter_save, ] <- z
          out_pi_ik[iter_save, , ] <- pi_ik
        }
        out_pi_mean <- out_pi_mean + pi_ik
      }

      if (verbose && (iter %% 500 == 0 || iter == 1)) {
        # elapsed <- as.numeric(difftime(Sys.time(), start_time, units = "secs"))
        # rate <- iter / elapsed
        cat(sprintf(
          "Iter %4d | LL=%.1f | acc_rate_nu=%.3f | acc_rate_beta=%.3f\n",
          iter, logliks[iter], acc_count_nu / iter, # min(acc_count_nu / iter), max(acc_count_nu / iter),
          acc_count_beta / iter # min(acc_count_beta / iter), max(acc_count_beta / iter)
        ))
      }
    }

    # finalize posterior mean of pi
    ##n_saved <- max(1, iter_save)
    out_pi_mean <- out_pi_mean / max(1, nsave) # average over saved iterations

    ret <- list(
      Beta_samples = out_beta,
      nu = out_nu,
      Sigma = out_Sigma,
      z_samples = out_z,
      pi_ik = out_pi_ik,
      pi_mean = out_pi_mean,
      estimate_nu = estimate_nu,
      burnin = burnin, niter = niter, thin = thin,
      loglik = logliks, loglik_individual = logliks_individual
    )
    class(ret) <- c("moewishart.bayes")
  }

  if (method == "em") {
    maxit <- niter
    ret <- moewishart.em(
      S_list, X, K, maxit, tol, verbose,
      init, estimate_nu, init_nu, ridge
    )
    ret$estimate_nu <- estimate_nu
    class(ret) <- c("moewishart.em")
  }

  return(ret)
}

# -- Internal help functions for full Bayesian--

# helper: compute pi_ik matrix (n x K) given Beta
softmax_rows <- function(L) {
  # L: n x K matrix of linear predictors
  m <- apply(L, 1, max)
  Ls <- L - m
  S <- exp(Ls)
  row_sums <- rowSums(S)
  S / row_sums
}
compute_pi_ik <- function(X, Beta) {
  L <- X %*% Beta # n x K
  # numeric stability: subtract row max
  rm <- apply(L, 1, max)
  Ls <- L - rm
  expL <- exp(Ls)
  expL / rowSums(expL)
}

# -- Internal function with EM algorithm for the Wishart mixture-of-experts model--

moewishart.em <- function(S_list, X, K, maxit = 200, tol = 1e-6, verbose = TRUE,
                          init = NULL, estimate_nu = FALSE, init_nu = NULL, ridge = 1e-8) {
  n <- length(S_list)
  p <- nrow(S_list[[1]])
  q <- ncol(X)

  # --- CORRECTION 1: BREAK SYMMETRY IN INITIALIZATION ---

  if (is.null(init) || is.null(init$beta)) {
    beta_vec <- rep(0, q * (K - 1))
  } else {
    beta_vec <- as.numeric(init$beta)
  }

  if (is.null(init) || is.null(init$Sigma)) {
    pooled <- Reduce("+", S_list) / n
    if (is.null(init_nu)) init_nu <- rep(p + 5, K)
    # Add slight jitter to initial Sigmas to ensure they aren't identical
    Sigma_list <- lapply(1:K, function(k) {
      (pooled / init_nu[k]) * (1 + runif(1, -0.1, 0.1))
    })
  } else {
    Sigma_list <- init$Sigma
  }

  if (is.null(init_nu)) init_nu <- rep(p + 5, K)
  nu_vec <- init_nu

  loglik_hist <- numeric(maxit)
  small_eps <- ridge

  # precompute log|S_i| for all i
  logdetS <- sapply(S_list, function(S) as.numeric(determinant(S, logarithm = TRUE)$modulus))

  # EM loop
  for (iter in 1:maxit) {
    # --- E-step ---
    pi_mat <- gating_probs_from_beta(beta_vec, q, K, X)
    log_f_mat <- matrix(NA, n, K)

    for (k in 1:K) {
      for (i in 1:n) {
        # Use precomputed logdetS to speed up
        log_f_mat[i, k] <- dWishart(S_list[[i]], nu_vec[k], Sigma_list[[k]],
          logdetS[i],
          logarithm = TRUE
        )
      }
    }

    log_post <- log(pi_mat + 1e-300) + log_f_mat
    row_max <- apply(log_post, 1, max)
    exp_shifted <- exp(log_post - row_max)
    row_sums <- rowSums(exp_shifted)
    gamma <- exp_shifted / row_sums

    loglik <- sum(row_max + log(row_sums))
    loglik_hist[iter] <- loglik

    if (verbose) cat(sprintf("Iter %3d  loglik = %.6f\n", iter, loglik))

    if (iter > 1 && abs(loglik - loglik_hist[iter - 1]) < tol) {
      if (verbose) cat("Converged by loglik tolerance.\n")
      break
    }

    # --- M-step ---
    an <- compute_A_nk(gamma, K, p, n, S_list)
    A_list <- an$A
    n_k <- an$n_k

    # Avoid division by zero if a cluster dies
    valid_k <- n_k > 1e-6

    for (k in 1:K) {
      if (valid_k[k]) {
        if (estimate_nu) {
          mean_logS_k <- sum(gamma[, k] * logdetS) / n_k[k]
          nu_old <- nu_vec[k]
          nu_vec[k] <- solve_nu(nu_old, A_list[[k]], n_k[k], mean_logS_k, p)
        }

        # Update Sigma using the NEW nu
        Sigma_new <- A_list[[k]] / (nu_vec[k] * n_k[k])
        Sigma_new <- Sigma_new + diag(small_eps, p)
        Sigma_list[[k]] <- (Sigma_new + t(Sigma_new)) / 2
      } else {
        # Reset empty component (optional strategy)
        if (verbose) cat(sprintf("Resetting component %d\n", k))
        nu_vec[k] <- p + 5
        Sigma_list[[k]] <- diag(1, p)
      }
    }

    # Update beta
    opt <- optim(beta_vec,
      fn = neg_wt_multinom, gr = grad_wt_multinom,
      q, K, X, # Further arguments to be passed to fn and gr
      method = "BFGS", gamma = gamma, control = list(maxit = 50)
    )
    beta_vec <- opt$par
  }

  Beta_mat <- matrix(0, q, K)
  if (K > 1) Beta_mat[, 1:(K - 1)] <- matrix(beta_vec, q, K - 1)
  colnames(Beta_mat) <- paste0("comp", 1:K)

  list(
    K = K, p = p, q = q, n = n,
    Beta = Beta_mat, Sigma = Sigma_list, nu = nu_vec,
    gamma = gamma, loglik = loglik_hist[1:iter], iter = iter
  )
}

# -- Internal help functions for EM algorithm--

# helper: log multivariate gamma derivative (psi_p) and its derivative
psi_p <- function(a, p) sum(digamma(a + (1 - (1:p)) / 2))

trigamma_p <- function(a, p) sum(trigamma(a + (1 - (1:p)) / 2))

compute_A_nk <- function(gamma, K, p, n, S_list) {
  n_k <- colSums(gamma)
  A_list <- vector("list", K)
  for (k in 1:K) {
    A <- matrix(0, p, p)
    # Vectorizing this sum is hard with list of matrices, loop is okay
    for (i in 1:n) A <- A + gamma[i, k] * S_list[[i]]
    A_list[[k]] <- A
  }
  list(A = A_list, n_k = n_k)
}
gating_probs_from_beta <- function(beta_vec, q, K, X) {
  B <- matrix(0, q, K)
  if (K > 1) {
    B[, 1:(K - 1)] <- matrix(beta_vec, q, K - 1)
  }
  eta <- X %*% B
  eta_max <- apply(eta, 1, max)
  exp_eta <- exp(eta - eta_max)
  denom <- rowSums(exp_eta)
  exp_eta / denom
}

neg_wt_multinom <- function(beta_vec, gamma, q, K, X) {
  pi_mat <- gating_probs_from_beta(beta_vec, q, K, X)
  # prevent log(0)
  ll <- sum(gamma * log(pi_mat + 1e-300))
  -ll
}

grad_wt_multinom <- function(beta_vec, gamma, q, K, X) {
  B <- matrix(0, q, K)
  if (K > 1) B[, 1:(K - 1)] <- matrix(beta_vec, q, K - 1)
  pi_mat <- gating_probs_from_beta(beta_vec, q, K, X)
  grad_mat <- matrix(0, q, K - 1)
  for (k in 1:(K - 1)) {
    # gradient of NEGATIVE likelihood is sum((pi - gamma) * X)
    diff <- pi_mat[, k] - gamma[, k]
    grad_mat[, k] <- colSums(X * diff)
  }
  as.numeric(grad_mat)
}

# --- CORRECTION 2: FIX NEWTON-RAPHSON DERIVATIVE ---
solve_nu <- function(init_nu, A_k, n_k, mean_logS_k, p, maxit = 20) {
  nu <- max(init_nu, p + 1.001)

  # OPTIMIZATION: Compute logdet(A) once outside the loop
  # A_k is constant during the nu update
  logdetA <- as.numeric(determinant(A_k, logarithm = TRUE)$modulus)

  for (it in 1:maxit) {
    # Use algebraic property: log|A / (n*nu)| = log|A| - p*log(n*nu)
    logdetSigma <- logdetA - p * log(n_k * nu)

    # Objective function
    lhs <- psi_p(nu / 2, p)
    rhs <- mean_logS_k - logdetSigma - p * log(2)
    fval <- lhs - rhs

    if (abs(fval) < 1e-6) break

    # Derivative
    df <- 0.5 * trigamma_p(nu / 2, p) - (p / nu)

    # Newton update
    nu_new <- nu - fval / df

    # Boundary check
    if (!is.finite(nu_new) || nu_new <= p + 1e-6) {
      nu_new <- (nu + (p + 1.0)) / 2
    }
    if (abs(nu_new - nu) < 1e-6) {
      nu <- nu_new
      break
    }
    nu <- nu_new
  }
  max(nu, p + 1.001)
}

#' @title CTRP drug response covariances data
#'
#' @description
#' The empirical per-drug dose-dose covariance from the 
#' Cancer Therapeutics Response Portal (CTRP v2, 2015) database 
#' \url{https://portals.broadinstitute.org/ctrp}. Based on a subset of 
#' n=374 drugs profiled at the same p=5 concentrations: 0.002, 0.016, 0.130, 
#' 1.000 and 8.300uM. Using replicate viability measurements (cell lines), we 
#' computed one p×p covariance matrix S(d) per drug, yielding n= 374 matrices.
#'
#' @examples
#' # Load the covariance data from the CTRP dataset
#' data("CTRP", package = "moewishart")
#' head(CTRP)
#'
"CTRP"

#' @title Information criteria for Wishart mixtures and MoE models
#'
#' @description
#' Compute AIC, BIC, and ICL for EM fits; and PSIS-LOO expected
#' log predictive density (elpd_loo) for Bayesian fits. Supports
#' \code{mixturewishart} (finite mixture) and \code{moewishart} (MoE with
#' covariates in gating).
#'
#' @name computeIC
#'
#' @importFrom utils tail
#' @importFrom loo loo
#'
#' @param fit A fitted object returned by \code{mixturewishart()} or
#'   \code{moewishart()}.
#'
#' @details
#' For EM fits:
#' \itemize{
#'   \item AIC: \eqn{\mathrm{AIC} = 2k - 2\ell},
#'         where \eqn{k} is the number of free parameters and
#'         \eqn{\ell} is the maximized log-likelihood (last EM iteration).
#'   \item BIC: \eqn{\mathrm{BIC} = k \log n - 2\ell}.
#'   \item ICL: \eqn{\mathrm{ICL} = \mathrm{BIC} + \sum_{i=1}^n
#'         \sum_{k=1}^K \tau_{ik}\log \tau_{ik}},
#'         i.e., BIC plus the entropy term (classification likelihood
#'         approximation).
#' }
#'
#' Parameter counting \eqn{k}:
#' \itemize{
#'   \item For \code{mixturewishart}:
#'         \eqn{k = (K-1) + K \cdot \frac{p(p+1)}{2} + K \cdot \mathbb{I}[\text{estimate }\nu]},
#'         where \eqn{(K-1)} are mixture weights, each \eqn{\Sigma_k} has
#'         \eqn{\frac{p(p+1)}{2}} free parameters, and \eqn{\nu_k} adds 1
#'         per component when estimated.
#'   \item For \code{moewishart}:
#'         \eqn{k = q\,(K-1) + K \cdot \frac{p(p+1)}{2} + K \cdot \mathbb{I}[\text{estimate }\nu]},
#'         where \eqn{q\,(K-1)} are gating regression coefficients
#'         (last class is reference with zero column).
#' }
#'
#' For Bayesian fits:
#' \itemize{
#'   \item elpd_loo: computed via \code{loo::loo} using per-observation
#'         log-likelihood draws. Requires \code{fit$loglik_individual}
#'         as a matrix of size \eqn{S \times n} (draws by observations),
#'         as produced by the provided samplers. Returns \code{elpd_loo},
#'         \code{se_elpd_loo}, \code{p_loo}, and \code{looic}.
#' }
#'
#' Notes:
#' \itemize{
#'   \item AIC/BIC/ICL are defined for MLE (EM) fits. For Bayesian fits,
#'         prefer elpd_loo (or WAIC) rather than AIC/BIC.
#'   \item The entropy term in ICL uses EM responsibilities
#'         \eqn{\tau_{ik}} (field \code{tau} for \code{mixturewishart},
#'         \code{gamma} for \code{moewishart}).
#' }
#'
#' @return
#' - For \code{method="em"}: a list with fields \code{AIC}, \code{BIC}, \code{ICL}.
#' - For \code{method="bayes"}: a list with fields ICL and elpd of class
#'   `"loo"` as returned by [loo::loo()] that contains fields `estimates`,
#'   `pointwise`, `diagnostics`
#'
#' @examples
#'
#' # Bayesian example (MoE)
#'
#' # simulate data
#' set.seed(123)
#' n <- 500 # subjects
#' p <- 2
#' # True gating coefficients (last column zero)
#' set.seed(123)
#' Xq <- 3
#' K <- 3
#' betas <- matrix(runif(Xq * K, -2, 2), nrow = Xq, ncol = K)
#' betas[, K] <- 0
#' dat <- simData(n, p,
#'   Xq = 3, K = 3, betas = betas,
#'   pis = c(0.35, 0.40, 0.25),
#'   nus = c(8, 16, 3)
#' )
#' set.seed(123)
#' fit <- moewishart(
#'   dat$S,
#'   X = cbind(1, dat$X), K = 3,
#'   mh_sigma = c(0.2, 0.1, 0.1), # RW-MH variances (length K)
#'   mh_beta = c(0.2, 0.2), # RW-MH variances (length K-1)
#'   niter = 100, burnin = 50
#' )
#' computeIC(fit)
#'
#' @export
computeIC <- function(fit) {
  if (inherits(fit, "mixturewishart.em") || inherits(fit, "moewishart.em")) {
    # Extract dimensions and responsibilities
    if (inherits(fit, "moewishart.em")) {
      # MoE fit returned by moewishart(..., method="em")
      n <- nrow(fit$gamma)
      K <- ncol(fit$gamma)
      p <- nrow(fit$Sigma[[1]])
      q <- nrow(fit$Beta)
      tau <- fit$gamma
      loglik <- tail(fit$loglik, 1)
      # Count parameters: gating q*(K-1), Sigma, optional nu
      k <- q * (K - 1) + K * (p * (p + 1) / 2) + fit$estimate_nu * K
    } else {
      # Mixture model fit returned by mixturewishart(..., method="em")
      n <- nrow(fit$tau)
      K <- ncol(fit$tau)
      p <- nrow(fit$Sigma[[1]])
      tau <- fit$tau
      loglik <- tail(fit$loglik, 1)
      # Count parameters: (K-1) weights, Sigma, optional nu
      k <- (K - 1) + K * (p * (p + 1) / 2) + fit$estimate_nu * K
    }
    AIC <- 2 * k - 2 * loglik
    BIC <- k * log(n) - 2 * loglik
    # ICL: BIC + sum_i sum_k tau_{ik} log tau_{ik}
    entropy_term <- sum(tau * log(tau), na.rm = TRUE)
    ICL <- BIC + entropy_term

    out <- list(AIC = AIC, BIC = BIC, ICL = ICL)
  }

  if (inherits(fit, "mixturewishart.bayes") ||
    inherits(fit, "moewishart.bayes")) {
    # Prepare MCMC output
    burnin <- fit$burnin
    niter <- fit$niter
    thin <- fit$thin
    log_lik <- fit$loglik_individual[((1 + burnin):niter) / thin, ]

    # Compute ICL
    n <- NCOL(log_lik)
    K <- dim(fit$pi_ik)[3]
    if (inherits(fit, "mixturewishart.bayes")) {
      q <- 0
    } else {
      q <- dim(fit$Beta_samples)[2]
    }
    kk <- K * (p * (p + 1) / 2 + 1) + (K - 1) * (q + 1)
    bic <- -2 * mean(log_lik) + log(n) * kk
    r_ik <- fit$pi_ik[-c(1:burnin), , ]
    r_ik <- apply(r_ik, c(2, 3), mean)
    icl <- bic - 2 * sum(r_ik * log(r_ik), na.rm = TRUE)

    out <- list()
    out$ICL <- icl

    # Use loo package
    if (!requireNamespace("loo", quietly = TRUE)) {
      stop("Package 'loo' is required. Please install.packages('loo').")
    }

    out$elpd <- loo::loo(log_lik)
  }

  return(out)
}

#' @title Simulate data from a Wishart mixture or mixture-of-experts model
#'
#' @description
#' Generate synthetic SPD matrices from either:
#' (i) a finite mixture of Wishart components with fixed mixing proportions, or
#' (ii) a mixture-of-experts (MoE) where mixing proportions depend on covariates
#'   via a softmax gating model.
#'
#' @name simData
#'
#' @importFrom stats rbinom rnorm runif rexp rgamma pnorm rWishart toeplitz
#'
#' @param n Integer. Number of observations to simulate.
#' @param p Integer. Dimension of the Wishart distribution (matrix size
#'   \eqn{p \times p}).
#' @param K Integer. Number of latent components. Required when
#'   \code{Xq > 0}. If \code{Xq = 0}, defaults to \code{length(pis)}.
#' @param Xq Integer. Number of covariates for the gating network
#'   (MoE case). If \code{Xq = 0}, a standard mixture (no covariates)
#'   is simulated.
#' @param betas Numeric matrix \eqn{Xq \times K} of gating coefficients
#'   used when \code{Xq > 0}. If \code{NULL}, random coefficients are
#'   generated and the last column is set to zero (reference class).
#' @param pis Numeric vector of length \eqn{K} giving fixed mixture
#'   proportions when \code{Xq = 0}. Ignored when \code{Xq > 0}.
#' @param nus Numeric vector length \eqn{K}, degrees of freedom
#'   \eqn{\nu_k} for each component (must exceed \eqn{p - 1}).
#' @param Sigma Optional list length \eqn{K} of SPD scale matrices
#'   \eqn{\Sigma_k} (each \eqn{p \times p}). If \code{NULL}, defaults are
#'   generated based on \code{K} and \code{p}.
#'
#'
#' @details
#' Models:
#' \itemize{
#'   \item Fixed mixture (no covariates, \code{Xq = 0}):
#'         \eqn{z_i \sim \mathrm{Categorical}(\pi)}, and
#'         \eqn{S_i \mid z_i=k \sim W_p(\nu_k, \Sigma_k)}.
#'   \item Mixture-of-experts (covariates, \code{Xq > 0}):
#'         Let \eqn{X_i \in \mathbb{R}^{Xq}}. The mixing weights are
#'         \eqn{\pi_{ik} = \Pr(z_i=k \mid X_i)} given by softmax regression
#'         \eqn{\pi_{ik} = \exp(X_i^\top B_k) / \sum_{j=1}^K
#'         \exp(X_i^\top B_j)}. Labels \eqn{z_i} are drawn from
#'         \eqn{\mathrm{Categorical}(\pi_i)} and
#'         \eqn{S_i \mid z_i=k \sim W_p(\nu_k, \Sigma_k)}.
#' }
#'
#' Simulation steps:
#' \enumerate{
#'   \item Construct \code{pis}:
#'         \itemize{
#'           \item If \code{Xq = 0}, replicate the provided \code{pis}
#'                 over \code{n} rows.
#'           \item If \code{Xq > 0}, generate \code{X} ~ N(0, I) and compute
#'                 softmax probabilities using \code{betas} (last column set
#'                 to zero by default identifiability).
#'         }
#'   \item If \code{Sigma} is not provided, create default \eqn{\Sigma_k}
#'         matrices (SPD) depending on \code{K} and \code{p}.
#'   \item Sample labels \eqn{z_i \sim \mathrm{Categorical}(\pi_i)}.
#'   \item Draw \eqn{S_i} from \eqn{W_p(\nu_{z_i}, \Sigma_{z_i})} via
#'         \code{rWishart}.
#' }
#'
#' Note that:
#' (i) in the MoE case, no intercept is automatically added to \code{X}.
#'         Use \code{Xq} to include desired covariates; the default
#'         \code{betas} is randomly generated with \code{betas[, K] = 0}, and
#' (ii) provided \code{Sigma} must be a list of SPD \eqn{p \times p}
#'         matrices. Provided \code{nus} must exceed \eqn{p - 1}.
#'
#'
#' @return A list with the following elements:
#' \itemize{
#'   \item \code{S}: list of length \code{n} of simulated SPD matrices
#'         \eqn{S_i}.
#'   \item \code{z}: integer vector length \code{n} of component labels.
#'   \item \code{nu}: numeric vector length \eqn{K} of degrees of freedom.
#'   \item \code{pi}: matrix \eqn{n \times K} of mixing probabilities
#'         (rows sum to \eqn{1}).
#'   \item \code{Sigma_list}: list length \eqn{K} of the scale matrices
#'         used for simulation.
#'   \item \code{X}: matrix \eqn{n \times Xq} of covariates if
#'         \code{Xq > 0}, otherwise \code{NULL}.
#'   \item \code{betas}: the gating coefficient matrix used when
#'         \code{Xq > 0}, otherwise \code{NULL}.
#' }
#'
#'
#'
#' @examples
#'
#' # simulate data from mixture model (no covariates)
#' set.seed(123)
#' n <- 200 # subjects
#' p <- 10
#' dat <- simData(n, p,
#'   K = 3,
#'   pis = c(0.35, 0.40, 0.25),
#'   nus = c(8, 12, 3)
#' )
#' str(dat)
#'
#' @export
simData <- function(n = 200, p = 2,
                    Xq = 0, K = NA,
                    betas = NULL,
                    pis = c(0.4, 0.6),
                    nus = c(8, 12),
                    Sigma = NULL) {
  # number of latent components
  if (Xq == 0) {
    K <- length(pis)
  } else {
    if (is.na(K)) {
      stop("Argument 'K' must be specified!")
    }
  }

  if (length(nus) != K) {
    stop("Arguments 'pis' and 'nus' must have the same length!")
  }

  # check betas matrix for MoE model
  if (Xq > 0 && (!is.null(betas))) {
    if (!is.matrix(betas)) {
      stop("Argument 'betas' must be a matrix!")
    }
    dims <- dim(betas)
    if (dims[1] != Xq || dims[2] != Xq) {
      stop("Argument 'betas' has wrong dimensions!")
    }
  }

  if (Xq > 0 && is.null(betas)) {
    betas <- matrix(runif(Xq * K, -2, 2), nrow = Xq, ncol = K)
    betas[, K] <- 0
  }

  # simulate covarites
  X <- NULL
  if (Xq > 0) {
    pis <- matrix(NA, nrow = n, ncol = K)
    X <- matrix(rnorm(n * Xq), nrow = n, ncol = Xq)
    expXb <- exp(X %*% betas)
    sumExpXb <- rowSums(expXb)
    pis <- apply(expXb, 2, function(xb_k) xb_k / sumExpXb)
  } else {
    pis <- matrix(pis, nrow = n, ncol = K, byrow = TRUE)
  }

  # define the scale matrix of the Wishart distribution
  if (is.null(Sigma)) {
    if (K == 2) {
      Sigma_list <- list(diag(c(1, 2)), matrix(c(2, 0.5, 0.5, 1), 2, 2))
    } else if (K == 3) {
      Sigma_list <- list(
        matrix(c(.5, .2, 0.2, 0.7), 2, 2),
        matrix(c(2.0, .6, 0.6, 1.5), 2, 2),
        matrix(c(4, .2, .2, 3), 2, 2)
      )
    } else {
      Sigma_list <- list()
      for (k in 1:K) {
        Sigma_list[[k]] <- rWishart(1, p, toeplitz((p:1) / p))[, , 1]
      }
    }
  } else {
    if (NROW(Sigma[[1]]) != p || NCOL(Sigma[[1]]) != p) {
      stop("The given 'Sigma' matrix has incorrect dimension!")
    }

    if (any(nus < p)) {
      stop("The given 'nus' has too small degrees of freedom!")
    }
    Sigma_list <- Sigma
  }

  S_list <- vector("list", n)
  z_true <- integer(n)
  for (i in 1:n) {
    k <- sample.int(K, 1, prob = pis[i, ])
    z_true[i] <- k
    # sample S ~ Wishart(nu_k, Sigma_k) using rWishart
    W <- rWishart(1, df = nus[k], Sigma = Sigma_list[[k]])[, , 1]
    S_list[[i]] <- W
  }

  list(
    S = S_list, z = z_true,
    nu = nus, pi = pis,
    Sigma_list = Sigma_list,
    X = X, betas = betas
  )
}

#' @title Dirichlet random sampling
#'
#' @description
#' Generate random draws from a Dirichlet distribution with parameter
#' vector \eqn{\alpha \in \mathbb{R}_+^K}. Each draw is a length-\eqn{K}
#' probability vector on the simplex.
#'
#' @name rdirichlet
#'
#' @importFrom stats rbinom rnorm runif rexp rgamma pnorm rWishart toeplitz
#'
#' @param n Integer. Number of independent Dirichlet draws to generate.
#' @param alpha Numeric vector of positive concentration parameters
#'   \eqn{\alpha = (\alpha_1,\dots,\alpha_K)}. Its length \eqn{K} defines
#'   the dimension of the simplex.
#'
#'
#' @details
#' Definition:
#'  If \eqn{Y_k \sim \mathrm{Gamma}(\alpha_k, 1)} independently for
#'         \eqn{k=1,\dots,K} (shape \eqn{\alpha_k}, rate 1), then the
#'         normalized vector \eqn{X_k = Y_k / \sum_{j=1}^K Y_j} follows
#'         \eqn{\mathrm{Dirichlet}(\alpha)}.
#'
#' Note that
#'  \code{alpha} must be a numeric vector with strictly positive entries.
#'
#'
#' @return A numeric matrix of size \eqn{n \times K}, where each row is
#'   an independent Dirichlet draw that sums to \eqn{1}.
#'
#'
#' @examples
#'
#' set.seed(123)
#' # 3-dimensional Dirichlet with asymmetric concentration
#' alpha <- c(2, 5, 3)
#' rdirichlet(5, alpha)
#'
#' @export
rdirichlet <- function(n, alpha) {
  l <- length(alpha)
  x <- matrix(rgamma(l * n, alpha), ncol = l, byrow = TRUE)
  x / rowSums(x)
}

---
title: "Introduction"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Introduction}
  \usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = TRUE)
options(rmarkdown.html_vignette.check_title = FALSE)
```

[![CRAN](https://www.r-pkg.org/badges/version/moewishart)](https://cran.r-project.org/package=moewishart)
[![r-universe](https://zhizuio.r-universe.dev/badges/moewishart)](https://zhizuio.r-universe.dev/moewishart)
[![R-CMD-check](https://github.com/zhizuio/moewishart/workflows/R-CMD-check/badge.svg)](https://github.com/zhizuio/moewishart/actions)
[![License](https://img.shields.io/badge/License-GPLv3-brightgreen.svg)](https://www.gnu.org/licenses/gpl-3.0)



This R-package `moewishart` provides maximum likelihood estimation (MLE) and Bayesian estimation for the **Wishart mixture model** and the **Wishart mixture-of-experts** (**MoE-Wishart**) model. 
It implements four different inference algorithms for the two model:

- mixture model of Wishart distributions:
    - EM algorithm for finding the MLE;
    - Bayesian approach using Gibbs-within-MH sampling algorithm.
- Mixture-of-Expert model, in which the gating probabilities depend on covariates:
   - EM-MoE algorithm for finding the MLE;
   - Bayesian-MoE approach using a Gibbs-within-MH sampling algorithm.  


## Installation

Install the latest released version from [CRAN](https://CRAN.R-project.org/package=moewishart):

```{r eval=FALSE}
install.packages("moewishart")
```

Install the latest development version from [GitHub](https://github.com/zhizuio/moewishart):

```{r eval=FALSE}
# library("devtools")
devtools::install_github("zhizuio/moewishart")
```

## Example


<br> 

Data simulation from a MoE-Wishart model:

- Sample size $n = 200$
- Dimension of the Wishart distribution $p = 2$
- Number of latent components $K = 3$
- $q=3$ gating covariates $\mathbf X = [x_{ij}] \in \mathbb R^{n\times q}$, $x_{ij}\sim\text{N}(0,1)$, $i=1,...,n$, $j=1,...,q$
- Fixed covariate effects $\boldsymbol\beta=[\boldsymbol\beta_1,...,\boldsymbol\beta_K] \in \mathbb R^{q\times K}$, with $\boldsymbol\beta_{K}=0$
- Probabilities of subpopulations $\boldsymbol\pi = [\pi_{ik}] \in \mathbb R^{n\times q}$, $\pi_{ik} = \exp(\mathbf X_i\boldsymbol\beta_k) / \sum_{l=1}^K\exp(\mathbf X_i\boldsymbol\beta_l)$, $k=1,...,K$
- Degrees of freedom $\boldsymbol\nu = (\nu_1,\nu_2,\nu_3) = (8, 12, 3)$
- Scale matrices of the Wishart distribution $\Sigma_1$, $\Sigma_2$, $\Sigma_3 \in \mathbb R^{p\times p}$
- Data $S_i \sim \pi_{i1}\text{Wishart}(\nu_1, \Sigma_1) + \pi_{i2}\text{Wishart}(\nu_2, \Sigma_2) + \pi_{i3}\text{Wishart}(\nu_3, \Sigma_3)$


### 1. Working model: Bayesian MoE-Wishart model

```{r, results='hide'}
library(moewishart)

n <- 200 # number of subjects
p <- 2 # dimension of covariance matrix
set.seed(123) # fix coefficients of underlying MoE model
Xq <- 3
K <- 3
betas <- matrix(runif(Xq * K, -2, 2), nrow = Xq, ncol = K)
betas[, K] <- 0

# simulate data
dat <- simData(n, p,
  Xq = 3, K = 3, betas = betas,
  pis = c(0.35, 0.40, 0.25),
  nus = c(8, 12, 3)
)

# fit Bayesian MoE-Wishart model
set.seed(123)
fit <- moewishart(
  dat$S,
  X = cbind(1, dat$X), K = 3,
  mh_sigma = c(0.2, 0.1, 0.2), # RW-MH variances (length K)
  mh_beta = c(0.3, 0.3), # RW-MH variances (length K-1)
  niter = 3000, burnin = 1000
)
```

<br> 

Posterior means for degrees of freedom (DoF) of Wishart distributions:

```{r}
burnin <- 1000
nu_mcmc <- fit$nu[-c(1:burnin), ]
colMeans(nu_mcmc)
```

<br> 
True DoF:

```{r}
dat$nu # true nu
```

<br> 
Posterior means for scale matrices of Wishart distributions:

```{r}
MoE_Sigma <- Reduce("+", fit$Sigma) / length(fit$Sigma)
MoE_Sigma
```

<br> 
Posterior means for gating coefficients:

```{r}
beta_mcmc <- fit$Beta_samples[-c(1:burnin), , ]
apply(beta_mcmc, c(2, 3), mean)
```


### 2. Working model: Bayesian Wishart mixture model

```{r, results='hide'}
# fit Bayesian Wishart mixture model
set.seed(123)
fit2 <- mixturewishart(
  dat$S,
  K = 3,
  mh_sigma = c(0.2, 0.1, 0.2), # RW-MH variances
  niter = 3000, burnin = 1000
)
```

<br> 
Posterior means for subpopulation probabilities:

```{r}
colMeans(fit2$pi[-c(1:burnin), ])
```

<br> 

Posterior means for DoF of Wishart distributions:

```{r}
colMeans(fit2$nu[-c(1:burnin), ])
```


### 3. Working model: MoE-Wishart model via EM algorithm

```{r}
# fit MoE-Wishart model via EM alg.
set.seed(123)
fit3 <- moewishart(
  dat$S,
  X = cbind(1, dat$X), K = 3,
  method = "em",
  niter = 3000
)
```

<br> 

EM estimates for DoF of Wishart distributions:

```{r}
fit3$nu
```

<br> 
EM estimates for Wishart scale matrices:

```{r}
fit3$Sigma
```

<br> 
EM estimates for gating coefficients:

```{r}
fit3$Beta
```


### 4. Working model: Wishart mixture model via EM algorithm

```{r}
# fit Wishart mixture model via EM alg.
set.seed(123)
fit4 <- mixturewishart(
  dat$S,
  K = 3,
  method = "em",
  niter = 3000
)
```

<br> 
EM estimates for DoF of Wishart distributions:

```{r}
fit4$nu
```

<br> 
EM estimate for Wishart scale matrices:

```{r}
fit4$Sigma
```


## Reference

Mai TT, Zhao Z (2026). 
Mixture-of-experts Wishart model for covariance matrices with an application to Cancer drug screening. 
[arXiv:2602.13888](https://doi.org/10.48550/arXiv.2602.13888).

---
title: "Introduction"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Introduction}
  \usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = TRUE)
options(rmarkdown.html_vignette.check_title = FALSE)
```

[![CRAN
status](https://www.r-pkg.org/badges/version/moewishart)](https://cran.r-project.org/package=moewishart)
[![r-universe](https://zhizuio.r-universe.dev/badges/moewishart)](https://zhizuio.r-universe.dev/moewishart)
[![R-CMD-check](https://github.com/zhizuio/moewishart/workflows/R-CMD-check/badge.svg)](https://github.com/zhizuio/moewishart/actions)
[![License](https://img.shields.io/badge/License-GPLv3-brightgreen.svg)](https://www.gnu.org/licenses/gpl-3.0)



This R-package `moewishart` provides maximum likelihood estimation (MLE) and Bayesian estimation for the **Wishart mixture model** and the **Wishart mixture-of-experts** (**MoE-Wishart**) model. 
It implements four different inference algorithms for the two model:

- mixture model of Wishart distributions:
    - EM algorithm for finding the MLE;
    - Bayesian approach using Gibbs-within-MH sampling algorithm.
- Mixture-of-Expert model, in which the gating probabilities depend on covariates:
   - EM-MoE algorithm for finding the MLE;
   - Bayesian-MoE approach using a Gibbs-within-MH sampling algorithm.  


## Installation

Install the latest released version from [CRAN](https://CRAN.R-project.org/package=moewishart):

```{r eval=FALSE}
install.packages("BayesSUR")
```

Install the latest development version from [GitHub](https://github.com/zhizuio/moewishart):

```{r eval=FALSE}
# library("devtools")
devtools::install_github("zhizuio/moewishart")
```

## Example


<br> 

Data simulation from a MoE-Wishart model:

- Sample size $n = 200$
- Dimension of the Wishart distribution $p = 2$
- Number of latent components $K = 3$
- $q=3$ gating covariates $\mathbf X = [x_{ij}] \in \mathbb R^{n\times q}$, $x_{ij}\sim\text{N}(0,1)$, $i=1,...,n$, $j=1,...,q$
- Fixed covariate effects $\boldsymbol\beta=[\boldsymbol\beta_1,...,\boldsymbol\beta_K] \in \mathbb R^{q\times K}$, with $\boldsymbol\beta_{K}=0$
- Probabilities of subpopulations $\boldsymbol\pi = [\pi_{ik}] \in \mathbb R^{n\times q}$, $\pi_{ik} = \exp(\mathbf X_i\boldsymbol\beta_k) / \sum_{l=1}^K\exp(\mathbf X_i\boldsymbol\beta_l)$, $k=1,...,K$
- Degrees of freedom $\boldsymbol\nu = (\nu_1,\nu_2,\nu_3) = (8, 12, 3)$
- Scale matrices of the Wishart distribution $\Sigma_1$, $\Sigma_2$, $\Sigma_3 \in \mathbb R^{p\times p}$
- Data $S_i \sim \pi_{i1}\text{Wishart}(\nu_1, \Sigma_1) + \pi_{i2}\text{Wishart}(\nu_2, \Sigma_2) + \pi_{i3}\text{Wishart}(\nu_3, \Sigma_3)$


### 1. Working model: Bayesian MoE-Wishart model

```{r, results='hide'}
library(moewishart)

n <- 200 # number of subjects
p <- 2 # dimension of covariance matrix
set.seed(123) # fix coefficients of underlying MoE model
Xq <- 3
K <- 3
betas <- matrix(runif(Xq * K, -2, 2), nrow = Xq, ncol = K)
betas[, K] <- 0

# simulate data
dat <- simData(n, p,
  Xq = 3, K = 3, betas = betas,
  pis = c(0.35, 0.40, 0.25),
  nus = c(8, 12, 3)
)

# fit Bayesian MoE-Wishart model
set.seed(123)
fit <- moewishart(
  dat$S,
  X = cbind(1, dat$X), K = 3,
  mh_sigma = c(0.2, 0.1, 0.2), # RW-MH variances (length K)
  mh_beta = c(0.3, 0.3), # RW-MH variances (length K-1)
  niter = 3000, burnin = 1000
)
```

<br> 

Posterior means for degrees of freedom (DoF) of Wishart distributions:

```{r}
burnin <- 1000
nu_mcmc <- fit$nu[-c(1:burnin), ]
colMeans(nu_mcmc)
```

<br> 
True DoF:

```{r}
dat$nu # true nu
```

<br> 
Posterior means for scale matrices of Wishart distributions:

```{r}
MoE_Sigma <- Reduce("+", fit$Sigma) / length(fit$Sigma)
MoE_Sigma
```

<br> 
Posterior means for gating coefficients:

```{r}
beta_mcmc <- fit$Beta_samples[-c(1:burnin), , ]
apply(beta_mcmc, c(2, 3), mean)
```


### 2. Working model: Bayesian Wishart mixture model

```{r, results='hide'}
# fit Bayesian Wishart mixture model
set.seed(123)
fit2 <- mixturewishart(
  dat$S,
  K = 3,
  mh_sigma = c(0.2, 0.1, 0.2), # RW-MH variances
  niter = 3000, burnin = 1000
)
```

<br> 
Posterior means for subpopulation probabilities:

```{r}
colMeans(fit2$pi[-c(1:burnin), ])
```

<br> 

Posterior means for DoF of Wishart distributions:

```{r}
colMeans(fit2$nu[-c(1:burnin), ])
```


### 3. Working model: MoE-Wishart model via EM algorithm

```{r}
# fit MoE-Wishart model via EM alg.
set.seed(123)
fit3 <- moewishart(
  dat$S,
  X = cbind(1, dat$X), K = 3,
  method = "em",
  niter = 3000
)
```

<br> 

EM estimates for DoF of Wishart distributions:

```{r}
fit3$nu
```

<br> 
EM estimates for Wishart scale matrices:

```{r}
fit3$Sigma
```

<br> 
EM estimates for gating coefficients:

```{r}
fit3$Beta
```


### 4. Working model: Wishart mixture model via EM algorithm

```{r}
# fit Wishart mixture model via EM alg.
set.seed(123)
fit4 <- mixturewishart(
  dat$S,
  K = 3,
  method = "em",
  niter = 3000
)
```

<br> 
EM estimates for DoF of Wishart distributions:

```{r}
fit4$nu
```

<br> 
EM estimate for Wishart scale matrices:

```{r}
fit4$Sigma
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotMCMC.R
\name{plotMCMC}
\alias{plotMCMC}
\title{Trace plot of parameters}
\usage{
plotMCMC(object, estimator = "nu", coeff.idx = 2)
}
\arguments{
\item{object}{returned object from Bayesian MoE or mixture model}

\item{estimator}{specified parameter for traceplot}

\item{coeff.idx}{choice of one covariate to show its coefficients in clusters}
}
\value{
A trace plot
}
\description{
Draw MCMC trace plot of parameters
}
\examples{

# simulate data
set.seed(123)
n <- 500 # subjects
p <- 2
# True gating coefficients (last column zero)
set.seed(123)
Xq <- 3
K <- 3
betas <- matrix(runif(Xq * K, -2, 2), nrow = Xq, ncol = K)
betas[, K] <- 0
dat <- simData(n, p,
  Xq = 3, K = 3, betas = betas,
  pis = c(0.35, 0.40, 0.25),
  nus = c(8, 16, 3)
)

set.seed(123)
fit <- moewishart(
  dat$S,
  X = cbind(1, dat$X), K = 3,
  mh_sigma = c(0.2, 0.1, 0.1), # RW-MH variances (length K)
  mh_beta = c(0.2, 0.2), # RW-MH variances (length K-1)
  niter = 500, burnin = 200
)

plotMCMC(fit, estimator = "nu")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sampleIW.R
\name{sampleIW}
\alias{sampleIW}
\title{Fast sampler for the inverse-Wishart distribution}
\usage{
sampleIW(nu, Psi_inv)
}
\arguments{
\item{nu}{Numeric. Degrees of freedom \eqn{\nu} of the inverse-Wishart
(must exceed \eqn{p - 1}).}

\item{Psi_inv}{Numeric \eqn{p \times p} SPD matrix equal to
\eqn{\Psi^{-1}}, the inverse of the inverse-Wishart scale matrix.}
}
\value{
A \eqn{p \times p} SPD matrix \eqn{S} distributed as
  \eqn{\mathcal{IW}_p(\nu, \Psi)}.
}
\description{
Draw a random sample from an inverse-Wishart distribution
\eqn{\mathcal{IW}_p(\nu, \Psi)} using the identity
\eqn{S \sim \mathcal{IW}_p(\nu, \Psi)} iff
\eqn{S^{-1} \sim W_p(\nu, \Psi^{-1})}. This implementation accepts
\eqn{\Psi^{-1}} directly for speed.
}
\details{
Sampling scheme:
\itemize{
  \item Sample \eqn{W \sim W_p(\nu, \Psi^{-1})} using \code{rWishart}.
  \item Return \eqn{S = W^{-1}}, which has
        \eqn{\mathcal{IW}_p(\nu, \Psi)}.
}

Parameterization:
\itemize{
  \item \eqn{\nu} is the degrees of freedom, must satisfy
        \eqn{\nu > p - 1}.
  \item \eqn{\Psi} is the SPD scale matrix of the inverse-Wishart.
        This function expects its inverse \eqn{\Psi^{-1}} as input
        for efficiency (avoids repeated matrix inversions).
}

Note that:
(i) internally calls \code{rWishart(1, df = nu, Sigma = Psi_inv)}, and
(ii) returns \code{solve(W)}; if numerical issues arise, consider
        adding a small ridge to \eqn{\Psi^{-1}} prior to sampling.
}
\examples{

set.seed(123)
p <- 3
# Construct an SPD scale matrix Psi
A <- matrix(rnorm(p * p), p, p)
Psi <- crossprod(A) + diag(p) * 0.5
Psi_inv <- solve(Psi)

# Draw one inverse-Wishart sample
S <- sampleIW(nu = p + 5, Psi_inv = Psi_inv)
S

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simData.R
\name{simData}
\alias{simData}
\title{Simulate data from a Wishart mixture or mixture-of-experts model}
\usage{
simData(
  n = 200,
  p = 2,
  Xq = 0,
  K = NA,
  betas = NULL,
  pis = c(0.4, 0.6),
  nus = c(8, 12),
  Sigma = NULL
)
}
\arguments{
\item{n}{Integer. Number of observations to simulate.}

\item{p}{Integer. Dimension of the Wishart distribution (matrix size
\eqn{p \times p}).}

\item{Xq}{Integer. Number of covariates for the gating network
(MoE case). If \code{Xq = 0}, a standard mixture (no covariates)
is simulated.}

\item{K}{Integer. Number of latent components. Required when
\code{Xq > 0}. If \code{Xq = 0}, defaults to \code{length(pis)}.}

\item{betas}{Numeric matrix \eqn{Xq \times K} of gating coefficients
used when \code{Xq > 0}. If \code{NULL}, random coefficients are
generated and the last column is set to zero (reference class).}

\item{pis}{Numeric vector of length \eqn{K} giving fixed mixture
proportions when \code{Xq = 0}. Ignored when \code{Xq > 0}.}

\item{nus}{Numeric vector length \eqn{K}, degrees of freedom
\eqn{\nu_k} for each component (must exceed \eqn{p - 1}).}

\item{Sigma}{Optional list length \eqn{K} of SPD scale matrices
\eqn{\Sigma_k} (each \eqn{p \times p}). If \code{NULL}, defaults are
generated based on \code{K} and \code{p}.}
}
\value{
A list with the following elements:
\itemize{
  \item \code{S}: list of length \code{n} of simulated SPD matrices
        \eqn{S_i}.
  \item \code{z}: integer vector length \code{n} of component labels.
  \item \code{nu}: numeric vector length \eqn{K} of degrees of freedom.
  \item \code{pi}: matrix \eqn{n \times K} of mixing probabilities
        (rows sum to \eqn{1}).
  \item \code{Sigma_list}: list length \eqn{K} of the scale matrices
        used for simulation.
  \item \code{X}: matrix \eqn{n \times Xq} of covariates if
        \code{Xq > 0}, otherwise \code{NULL}.
  \item \code{betas}: the gating coefficient matrix used when
        \code{Xq > 0}, otherwise \code{NULL}.
}
}
\description{
Generate synthetic SPD matrices from either:
(i) a finite mixture of Wishart components with fixed mixing proportions, or
(ii) a mixture-of-experts (MoE) where mixing proportions depend on covariates
  via a softmax gating model.
}
\details{
Models:
\itemize{
  \item Fixed mixture (no covariates, \code{Xq = 0}):
        \eqn{z_i \sim \mathrm{Categorical}(\pi)}, and
        \eqn{S_i \mid z_i=k \sim W_p(\nu_k, \Sigma_k)}.
  \item Mixture-of-experts (covariates, \code{Xq > 0}):
        Let \eqn{X_i \in \mathbb{R}^{Xq}}. The mixing weights are
        \eqn{\pi_{ik} = \Pr(z_i=k \mid X_i)} given by softmax regression
        \eqn{\pi_{ik} = \exp(X_i^\top B_k) / \sum_{j=1}^K
        \exp(X_i^\top B_j)}. Labels \eqn{z_i} are drawn from
        \eqn{\mathrm{Categorical}(\pi_i)} and
        \eqn{S_i \mid z_i=k \sim W_p(\nu_k, \Sigma_k)}.
}

Simulation steps:
\enumerate{
  \item Construct \code{pis}:
        \itemize{
          \item If \code{Xq = 0}, replicate the provided \code{pis}
                over \code{n} rows.
          \item If \code{Xq > 0}, generate \code{X} ~ N(0, I) and compute
                softmax probabilities using \code{betas} (last column set
                to zero by default identifiability).
        }
  \item If \code{Sigma} is not provided, create default \eqn{\Sigma_k}
        matrices (SPD) depending on \code{K} and \code{p}.
  \item Sample labels \eqn{z_i \sim \mathrm{Categorical}(\pi_i)}.
  \item Draw \eqn{S_i} from \eqn{W_p(\nu_{z_i}, \Sigma_{z_i})} via
        \code{rWishart}.
}

Note that:
(i) in the MoE case, no intercept is automatically added to \code{X}.
        Use \code{Xq} to include desired covariates; the default
        \code{betas} is randomly generated with \code{betas[, K] = 0}, and
(ii) provided \code{Sigma} must be a list of SPD \eqn{p \times p}
        matrices. Provided \code{nus} must exceed \eqn{p - 1}.
}
\examples{

# simulate data from mixture model (no covariates)
set.seed(123)
n <- 200 # subjects
p <- 10
dat <- simData(n, p,
  K = 3,
  pis = c(0.35, 0.40, 0.25),
  nus = c(8, 12, 3)
)
str(dat)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmvgamma.R
\name{lmvgamma}
\alias{lmvgamma}
\title{Log multivariate gamma function}
\usage{
lmvgamma(a, p)
}
\arguments{
\item{a}{Numeric. Argument of \eqn{\Gamma_p(\cdot)} (often \eqn{\nu/2}
in Wishart contexts).}

\item{p}{Integer. Dimension \eqn{p} of the multivariate gamma.}
}
\value{
A numeric scalar equal to \eqn{\log \Gamma_p(a)}.
}
\description{
Compute the log of the multivariate gamma function \eqn{\log \Gamma_p(a)}
for dimension \eqn{p} and parameter \eqn{a}.
}
\details{
The multivariate gamma function \eqn{\Gamma_p(a)} is defined by:
\deqn{
  \Gamma_p(a)
  = \pi^{\,p(p-1)/4} \prod_{j=1}^{p} \Gamma\!\left(a + \frac{1-j}{2}\right).
}

Constraints: (i) \eqn{p \in \{1,2,\dots\}} (positive integer), and
  (ii) \eqn{a > (p-1)/2} to keep all gamma terms finite (as in the Wishart
  normalization constant).
}
\examples{

# Dimension
p <- 3
# Evaluate log multivariate gamma at a = nu/2
nu <- p + 5
lmvgamma(a = nu / 2, p = p)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rdirichlet.R
\name{rdirichlet}
\alias{rdirichlet}
\title{Dirichlet random sampling}
\usage{
rdirichlet(n, alpha)
}
\arguments{
\item{n}{Integer. Number of independent Dirichlet draws to generate.}

\item{alpha}{Numeric vector of positive concentration parameters
\eqn{\alpha = (\alpha_1,\dots,\alpha_K)}. Its length \eqn{K} defines
the dimension of the simplex.}
}
\value{
A numeric matrix of size \eqn{n \times K}, where each row is
  an independent Dirichlet draw that sums to \eqn{1}.
}
\description{
Generate random draws from a Dirichlet distribution with parameter
vector \eqn{\alpha \in \mathbb{R}_+^K}. Each draw is a length-\eqn{K}
probability vector on the simplex.
}
\details{
Definition:
 If \eqn{Y_k \sim \mathrm{Gamma}(\alpha_k, 1)} independently for
        \eqn{k=1,\dots,K} (shape \eqn{\alpha_k}, rate 1), then the
        normalized vector \eqn{X_k = Y_k / \sum_{j=1}^K Y_j} follows
        \eqn{\mathrm{Dirichlet}(\alpha)}.

Note that
 \code{alpha} must be a numeric vector with strictly positive entries.
}
\examples{

set.seed(123)
# 3-dimensional Dirichlet with asymmetric concentration
alpha <- c(2, 5, 3)
rdirichlet(5, alpha)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixturewishart.R
\name{mixturewishart}
\alias{mixturewishart}
\title{EM/Bayesian estimation for Wishart mixture model}
\usage{
mixturewishart(
  S_list,
  K,
  niter = 3000,
  burnin = 1000,
  method = "bayes",
  thin = 1,
  alpha = NULL,
  nu0 = NULL,
  Psi0 = NULL,
  init_pi = NULL,
  init_nu = NULL,
  init_Sigma = NULL,
  marginal.z = TRUE,
  estimate_nu = TRUE,
  nu_prior_a = 2,
  nu_prior_b = 0.1,
  mh_sigma = 1,
  n_restarts = 3,
  restart_iters = 20,
  tol = 1e-06,
  verbose = TRUE
)
}
\arguments{
\item{S_list}{List of length \eqn{n} of SPD matrices, each \eqn{p \times p}.
These are the observed matrices modeled by a mixture of Wisharts.}

\item{K}{Integer. Number of mixture components.}

\item{niter}{Integer. Total iterations. Bayesian mode: total MCMC
iterations (including burn-in). EM mode: maximum EM iterations
(alias to \code{maxiter}).}

\item{burnin}{Integer. Number of burn-in iterations (Bayesian mode).}

\item{method}{Character; one of \code{c("bayes","em")}. Selects sampler
or optimizer.}

\item{thin}{Integer. Thinning interval for saving draws (Bayesian).}

\item{alpha}{Numeric vector length \eqn{K} (Dirichlet prior on
\eqn{\pi}) or \code{NULL} to default to \code{rep(1, K)} (Bayesian).}

\item{nu0}{Numeric. Inverse-Wishart prior df for \eqn{\Sigma_k}
(Bayesian). Default: \eqn{p + 2}.}

\item{Psi0}{Numeric \eqn{p \times p} SPD matrix. Inverse-Wishart prior
scale for \eqn{\Sigma_k} (Bayesian). Default: \code{diag(p)}.}

\item{init_pi}{Optional numeric vector length \eqn{K} summing to
\eqn{1}. EM initialization for mixture weights. If \code{NULL},
random or data-driven initialization is used.}

\item{init_nu}{Optional numeric vector length \eqn{K} of initial
degrees of freedom. Used in both modes.}

\item{init_Sigma}{Optional list of \eqn{K} SPD matrices (each
\eqn{p \times p}). EM initialization for \eqn{\Sigma_k}.}

\item{marginal.z}{Logical. If \code{TRUE}, integrates out \eqn{\pi}
when sampling \eqn{z} (collapsed step) in Bayesian mode. If
\code{FALSE}, samples \eqn{z} conditional on current \eqn{\pi}.}

\item{estimate_nu}{Logical. If \code{TRUE}, estimate/update
\eqn{\nu_k} (MH in Bayesian mode; Newton/EM in EM). If
\code{FALSE}, \eqn{\nu_k} are fixed.}

\item{nu_prior_a}{Numeric. Prior hyperparameter \eqn{a} for
\eqn{\nu_k} (Bayesian), used when \code{estimate_nu = TRUE}.}

\item{nu_prior_b}{Numeric. Prior hyperparameter \eqn{b} for
\eqn{\nu_k} (Bayesian), used when \code{estimate_nu = TRUE}.}

\item{mh_sigma}{Numeric scalar or length-\eqn{K} vector. Proposal sd
for MH updates on \eqn{\log(\nu_k)} (Bayesian, when estimating
\eqn{\nu}).}

\item{n_restarts}{Integer. Number of random restarts for EM. Ignored
in Bayesian mode.}

\item{restart_iters}{Integer. Number of short EM iterations per
restart used to select a good initialization. Ignored in Bayesian
mode.}

\item{tol}{Numeric. Convergence tolerance on absolute change of
log-likelihood (EM), also used internally elsewhere.}

\item{verbose}{Logical. If \code{TRUE}, print progress information.}
}
\value{
A list whose structure depends on \code{method}:
\itemize{
  \item For \code{method = "bayes"}:
    \itemize{
      \item \code{pi_ik}: array (\code{nsave} x \code{n} x \code{K}),
            saved per-observation weights.
      \item \code{pi}: matrix (\code{nsave} x \code{K}), saved mixture
            proportions.
      \item \code{nu}: matrix (\code{nsave} x \code{K}), saved degrees-
            of-freedom.
      \item \code{Sigma}: list of length \code{nsave}; each is an
            array (\eqn{p \times p \times K}) of \eqn{\Sigma_k} draws.
      \item \code{z}: matrix (\code{nsave} x \code{n}), saved
            allocations.
      \item \code{sigma_posterior_mean}: array (\eqn{p \times p \times
            K}), posterior mean of \eqn{\Sigma_k}.
      \item \code{loglik}: numeric vector (length \code{niter}), log-
            likelihood trace.
      \item \code{loglik_individual}: matrix (\code{niter} x
            \code{n}), per-observation log-likelihood.
    }
  \item For \code{method = "em"}:
    \itemize{
      \item \code{pi}: numeric vector length \eqn{K}, mixture
            proportions.
      \item \code{Sigma}: list length \code{K}, each a \eqn{p \times p}
            SPD matrix.
      \item \code{nu}: numeric vector length \code{K}, degrees-of-
            freedom.
      \item \code{tau}: matrix (\eqn{n \times K}), responsibilities.
      \item \code{loglik}: numeric vector, log-likelihood per EM
            iteration.
      \item \code{iterations}: integer, number of EM iterations
            performed.
    }
}
}
\description{
Fit finite mixtures of Wishart-distributed SPD matrices using either a
Bayesian sampler or the EM algorithm. The input \code{S_list} is a list
of \eqn{p \times p} SPD matrices. Under component \eqn{k},
\eqn{S_i \mid z_i=k \sim W_p(\nu_k, \Sigma_k)} with degrees of freedom
\eqn{\nu_k} and SPD scale matrix \eqn{\Sigma_k}. Mixture weights
\eqn{\pi_k} sum to \eqn{1}.
}
\details{
Mixture mixture model:
 \eqn{p(S_i) = \sum_{k=1}^K \pi_k \, f_W(S_i \mid \nu_k, \Sigma_k)}.

Algorithms:
\enumerate{
  \item \code{method = "bayes"}: Samples latent labels \eqn{z}, weights
        \eqn{\pi}, component scales \eqn{\Sigma_k}, and optionally
        \eqn{\nu_k}. Uses a Dirichlet prior for \eqn{\pi}, inverse-
        Wishart prior for \eqn{\Sigma_k}, and a prior on \eqn{\nu_k}
        when \code{estimate_nu = TRUE}. Degrees-of-freedom are updated
        via MH on \eqn{\log(\nu_k)} with proposal sd \code{mh_sigma}.
        Can integrate out \eqn{\pi} when sampling \eqn{z} if
        \code{marginal.z = TRUE}.
  \item \code{method = "em"}: Maximizes the observed-data log-
        likelihood via EM. The E-step computes responsibilities via
        Wishart log-densities. The M-step updates \eqn{\pi_k} and
        \eqn{\Sigma_k}; optionally updates \eqn{\nu_k} when
        \code{estimate_nu = TRUE}. Supports multiple random restarts.
}

Note that
(i) All matrices in \code{S_list} must be SPD. Small ridge terms may be
added internally for stability, and
(ii) Multiple EM restarts are recommended for robustness on difficult datasets.
}
\examples{

# simulate data
set.seed(123)
n <- 500 # subjects
p <- 2
dat <- simData(n, p,
  K = 3,
  pis = c(0.35, 0.40, 0.25),
  nus = c(8, 16, 3)
)

set.seed(123)
fit <- mixturewishart(
  dat$S,
  K = 3,
  mh_sigma = c(0.2, 0.1, 0.1), # tune this for MH acceptance 20-40\%
  niter = 100, burnin = 50
)

# Posterior means for degrees of freedom of Wishart distributions:
nu_mcmc <- fit$nu[-c(1:fit$burnin), ]
colMeans(nu_mcmc)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeIC.R
\name{computeIC}
\alias{computeIC}
\title{Information criteria for Wishart mixtures and MoE models}
\usage{
computeIC(fit)
}
\arguments{
\item{fit}{A fitted object returned by \code{mixturewishart()} or
\code{moewishart()}.}
}
\value{
- For \code{method="em"}: a list with fields \code{AIC}, \code{BIC}, \code{ICL}.
- For \code{method="bayes"}: a list with fields ICL and elpd of class
  `"loo"` as returned by [loo::loo()] that contains fields `estimates`,
  `pointwise`, `diagnostics`
}
\description{
Compute AIC, BIC, and ICL for EM fits; and PSIS-LOO expected
log predictive density (elpd_loo) for Bayesian fits. Supports
\code{mixturewishart} (finite mixture) and \code{moewishart} (MoE with
covariates in gating).
}
\details{
For EM fits:
\itemize{
  \item AIC: \eqn{\mathrm{AIC} = 2k - 2\ell},
        where \eqn{k} is the number of free parameters and
        \eqn{\ell} is the maximized log-likelihood (last EM iteration).
  \item BIC: \eqn{\mathrm{BIC} = k \log n - 2\ell}.
  \item ICL: \eqn{\mathrm{ICL} = \mathrm{BIC} + \sum_{i=1}^n
        \sum_{k=1}^K \tau_{ik}\log \tau_{ik}},
        i.e., BIC plus the entropy term (classification likelihood
        approximation).
}

Parameter counting \eqn{k}:
\itemize{
  \item For \code{mixturewishart}:
        \eqn{k = (K-1) + K \cdot \frac{p(p+1)}{2} + K \cdot \mathbb{I}[\text{estimate }\nu]},
        where \eqn{(K-1)} are mixture weights, each \eqn{\Sigma_k} has
        \eqn{\frac{p(p+1)}{2}} free parameters, and \eqn{\nu_k} adds 1
        per component when estimated.
  \item For \code{moewishart}:
        \eqn{k = q\,(K-1) + K \cdot \frac{p(p+1)}{2} + K \cdot \mathbb{I}[\text{estimate }\nu]},
        where \eqn{q\,(K-1)} are gating regression coefficients
        (last class is reference with zero column).
}

For Bayesian fits:
\itemize{
  \item elpd_loo: computed via \code{loo::loo} using per-observation
        log-likelihood draws. Requires \code{fit$loglik_individual}
        as a matrix of size \eqn{S \times n} (draws by observations),
        as produced by the provided samplers. Returns \code{elpd_loo},
        \code{se_elpd_loo}, \code{p_loo}, and \code{looic}.
}

Notes:
\itemize{
  \item AIC/BIC/ICL are defined for MLE (EM) fits. For Bayesian fits,
        prefer elpd_loo (or WAIC) rather than AIC/BIC.
  \item The entropy term in ICL uses EM responsibilities
        \eqn{\tau_{ik}} (field \code{tau} for \code{mixturewishart},
        \code{gamma} for \code{moewishart}).
}
}
\examples{

# Bayesian example (MoE)

# simulate data
set.seed(123)
n <- 500 # subjects
p <- 2
# True gating coefficients (last column zero)
set.seed(123)
Xq <- 3
K <- 3
betas <- matrix(runif(Xq * K, -2, 2), nrow = Xq, ncol = K)
betas[, K] <- 0
dat <- simData(n, p,
  Xq = 3, K = 3, betas = betas,
  pis = c(0.35, 0.40, 0.25),
  nus = c(8, 16, 3)
)
set.seed(123)
fit <- moewishart(
  dat$S,
  X = cbind(1, dat$X), K = 3,
  mh_sigma = c(0.2, 0.1, 0.1), # RW-MH variances (length K)
  mh_beta = c(0.2, 0.2), # RW-MH variances (length K-1)
  niter = 100, burnin = 50
)
computeIC(fit)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CTRP.R
\docType{data}
\name{CTRP}
\alias{CTRP}
\title{CTRP drug response covariances data}
\format{
An object of class \code{list} of length 374.
}
\usage{
CTRP
}
\description{
The empirical per-drug dose-dose covariance from the 
Cancer Therapeutics Response Portal (CTRP v2, 2015) database 
\url{https://portals.broadinstitute.org/ctrp}. Based on a subset of 
n=374 drugs profiled at the same p=5 concentrations: 0.002, 0.016, 0.130, 
1.000 and 8.300uM. Using replicate viability measurements (cell lines), we 
computed one p×p covariance matrix S(d) per drug, yielding n= 374 matrices.
}
\examples{
# Load the covariance data from the CTRP dataset
data("CTRP", package = "moewishart")
head(CTRP)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/moewishart.R
\name{moewishart}
\alias{moewishart}
\title{EM/Bayesian estimation for Wishart MoE model}
\usage{
moewishart(
  S_list,
  X,
  K,
  niter = 3000,
  burnin = 1000,
  method = "bayes",
  thin = 1,
  nu0 = NULL,
  Psi0 = NULL,
  init_nu = NULL,
  estimate_nu = TRUE,
  nu_prior_a = 2,
  nu_prior_b = 0.1,
  mh_sigma = 0.1,
  mh_beta = 0.05,
  sigma_beta = 10,
  init = NULL,
  tol = 1e-06,
  ridge = 1e-08,
  verbose = TRUE
)
}
\arguments{
\item{S_list}{List of length \eqn{n} of SPD matrices, each \eqn{p \times p}.
These are the observed responses modeled by the MoE.}

\item{X}{Numeric matrix \eqn{n \times q} of covariates for the gating
network. Include an intercept column if desired.}

\item{K}{Integer. Number of mixture components (experts).}

\item{niter}{Integer. Total iterations. Bayesian mode: total MCMC
iterations (including burn-in). EM mode: maximum EM iterations.}

\item{burnin}{Integer. Number of burn-in iterations (Bayesian mode).}

\item{method}{Character; one of \code{c("bayes", "em")}. Selects
sampler or optimizer.}

\item{thin}{Integer. Thinning interval for saving draws (Bayesian).}

\item{nu0}{Numeric. Inverse-Wishart prior df for \eqn{\Sigma_k}
(Bayesian). Default: \eqn{p + 2} if \code{NULL}.}

\item{Psi0}{Numeric \eqn{p \times p} SPD matrix. Inverse-Wishart prior
scale for \eqn{\Sigma_k} (Bayesian). Default: \code{diag(p)} if
\code{NULL}.}

\item{init_nu}{Optional numeric vector length \eqn{K} of initial dfs
\eqn{\nu_k}. Used for initialization.}

\item{estimate_nu}{Logical. If \code{TRUE}, estimate \eqn{\nu_k}
(MH in Bayesian; Newton/EM in EM). If \code{FALSE}, keep \eqn{\nu_k}
fixed at \code{init_nu}.}

\item{nu_prior_a}{Numeric. Prior hyperparameter \eqn{a} for \eqn{\nu_k}
(Bayesian), used when \code{estimate_nu = TRUE}.}

\item{nu_prior_b}{Numeric. Prior hyperparameter \eqn{b} for \eqn{\nu_k}
(Bayesian), used when \code{estimate_nu = TRUE}.}

\item{mh_sigma}{Numeric scalar or length-\eqn{K} vector. Proposal sd
for MH updates on \eqn{\log(\nu_k)} (Bayesian, when estimating
\eqn{\nu}).}

\item{mh_beta}{Numeric scalar or length-\eqn{K-1} vector. Proposal sd
for MH updates of the free \eqn{B} columns (Bayesian).}

\item{sigma_beta}{Numeric. Prior sd of the Gaussian prior on \eqn{B}
(Bayesian).}

\item{init}{Optional list with fields for EM initialization, e.g.,
\code{beta}, \code{Sigma}, \code{nu}. See return structure.}

\item{tol}{Numeric. Convergence tolerance on absolute change of
log-likelihood (EM), also used internally.}

\item{ridge}{Numeric. Small diagonal ridge added to \eqn{\Sigma_k}
updates in EM for numerical stability.}

\item{verbose}{Logical. If \code{TRUE}, print progress information.}
}
\value{
A list whose fields depend on \code{method}:
\itemize{
  \item For \code{method = "bayes"}:
    \itemize{
      \item \code{Beta_samples}: array (\code{nsave} x \code{q} x
            \code{K}), saved draws of \eqn{B} (last column zero).
      \item \code{nu}: matrix (\code{nsave} x \code{K}), draws
            of \eqn{\nu_k}.
      \item \code{Sigma}: list of length \code{nsave}; each
            element is an array (\eqn{p \times p \times K}) of
            \eqn{\Sigma_k} draws.
      \item \code{z_samples}: matrix (\code{nsave} x \code{n}), draws
            of allocations.
      \item \code{pi_ik}: array (\code{nsave} x \code{n} x \code{K}),
            per-observation gating probabilities.
      \item \code{pi_mean}: matrix (\code{n} x \code{K}), posterior
            mean of gating probabilities.
      \item \code{loglik}: numeric vector (length \code{niter}),
            log-likelihood trace.
      \item \code{loglik_individual}: matrix (\code{niter} x
            \code{n}), per-observation log-likelihood.
    }
  \item For \code{method = "em"}:
    \itemize{
      \item \code{K, p, q, n}: problem dimensions.
      \item \code{Beta}: matrix (\eqn{q \times K}), gating coefficients
            with last column zero (reference class).
      \item \code{Sigma}: list length \code{K}, each a \eqn{p \times p}
            SPD matrix (scale).
      \item \code{nu}: numeric vector length \code{K}, degrees of
            freedom.
      \item \code{gamma}: matrix (\eqn{n \times K}), final
            responsibilities.
      \item \code{loglik}: numeric vector, log-likelihood by EM
            iteration.
      \item \code{iter}: integer, number of EM iterations performed.
    }
}
}
\description{
Fit a mixture-of-experts model for symmetric positive-definite (SPD)
matrices with covariate-dependent mixing proportions (gating network).
Components are Wishart-distributed. Supports Bayesian sampling and
EM-based maximum-likelihood estimation.
}
\details{
MoE-Wishart Model:
\itemize{
  \item Observation: \eqn{S_i} is a \eqn{p \times p} SPD matrix. Given
        allocation \eqn{z_i=k}, \eqn{S_i \mid z_i \sim W_p(\nu_k,
        \Sigma_k)} with df \eqn{\nu_k} and scale \eqn{\Sigma_k}.
  \item Gating (MoE): Let \eqn{X_i} be \eqn{q}-dimensional covariates.
        Mixing weights \eqn{\pi_{ik} = \Pr(z_i=k \mid X_i)} follow a
        softmax regression:
        \eqn{\pi_{ik} = \exp(\eta_{ik})/\sum_{j=1}^K \exp(\eta_{ij})},
        where \eqn{\eta_i = X_i^\top B}, \eqn{B} is
        \eqn{q \times K}. Identifiability: last column of \eqn{B}
        is fixed to zero.
}

Algorithms:
\enumerate{
  \item Bayesian (\code{method = "bayes"}): Metropolis-within-Gibbs
        sampler for \eqn{z}, \eqn{\Sigma_k}, optional \eqn{\nu_k}, and
        \eqn{B}. Gaussian priors on \eqn{B} with sd
        \code{sigma_beta}. Proposals use \code{mh_sigma} for
        \eqn{\log(\nu_k)} and \code{mh_beta} for \eqn{B}.
  \item EM (\code{method = "em"}): E-step responsibilities using
        Wishart log-densities and softmax gating. M-step updates
        \eqn{\Sigma_k}, optional \eqn{\nu_k}, and \eqn{B} via
        weighted multinomial logistic regression (BFGS).
}

Note that:
(i) include an intercept column in \code{X}; none is added by default, and
(ii) all \code{S_list} elements must be SPD. A small \code{ridge} may be
added for stability.
}
\examples{

# simulate data
set.seed(123)
n <- 500 # subjects
p <- 2
# True gating coefficients (last column zero)
set.seed(123)
Xq <- 3
K <- 3
betas <- matrix(runif(Xq * K, -2, 2), nrow = Xq, ncol = K)
betas[, K] <- 0
dat <- simData(n, p,
  Xq = 3, K = 3, betas = betas,
  pis = c(0.35, 0.40, 0.25),
  nus = c(8, 16, 3)
)

set.seed(123)
fit <- moewishart(
  dat$S,
  X = cbind(1, dat$X), K = 3,
  mh_sigma = c(0.2, 0.1, 0.1), # RW-MH variances (length K)
  mh_beta = c(0.2, 0.2), # RW-MH variances (length K-1)
  niter = 500, burnin = 200
)

# Posterior means for degrees of freedom of Wishart distributions:
nu_mcmc <- fit$nu[-c(1:fit$burnin), ]
colMeans(nu_mcmc)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dWishart.R
\name{dWishart}
\alias{dWishart}
\title{density of Wishart distribution}
\usage{
dWishart(S, nu, Sigma, detS_val = NULL, logarithm = TRUE)
}
\arguments{
\item{S}{Numeric \eqn{p \times p} SPD matrix at which to evaluate the
density.}

\item{nu}{Numeric. Degrees of freedom \eqn{\nu} (must exceed \eqn{p-1}).}

\item{Sigma}{Numeric \eqn{p \times p} SPD scale matrix \eqn{\Sigma}.}

\item{detS_val}{Optional numeric. Precomputed \eqn{\log|S|} to reuse;
if \code{NULL}, it is computed internally.}

\item{logarithm}{Logical. If \code{TRUE}, return log-density;
otherwise return density.}
}
\value{
A numeric scalar: the log-density if \code{logarithm = TRUE},
  otherwise the density.
}
\description{
Compute the (log) density of a \eqn{p}-dimensional Wishart distribution
\eqn{W_p(\nu, \Sigma)} at an SPD matrix \eqn{S}. Returns either the
log-density or the density depending on \code{logarithm}.
}
\details{
Let \eqn{S \sim W_p(\nu, \Sigma)} with degrees of freedom \eqn{\nu}
and scale matrix \eqn{\Sigma} (SPD). The density is:
\deqn{
  f(S \mid \nu, \Sigma) =
  \frac{|S|^{(\nu - p - 1)/2}\,
        \exp\{-\tfrac{1}{2}\,\mathrm{tr}(\Sigma^{-1}S)\}}
       {2^{\nu p/2}\,|\Sigma|^{\nu/2}\,\Gamma_p(\nu/2)},
}
where \eqn{\Gamma_p(\cdot)} is the multivariate gamma function and
\eqn{p} is the dimension.

Note that
(i) \code{detS_val} can be supplied to avoid recomputing \eqn{\log|S|},
 which is useful inside EM/MCMC loops, and
 (ii) small diagonal jitter is added internally to \eqn{S} and \eqn{\Sigma}
 when computing determinants or solves for numerical stability.

Constraints: (i) \eqn{S} and \eqn{\Sigma} must be SPD, and (ii) the Wishart
requires \eqn{\nu > p - 1}.
}
\examples{

set.seed(123)
p <- 3
# Construct an SPD Sigma
A <- matrix(rnorm(p * p), p, p)
Sigma <- crossprod(A) + diag(p) * 0.5
# Draw a Wishart matrix using base stats::rWishart()
W <- drop(rWishart(1, df = p + 5, Sigma = Sigma))
# Evaluate log-density at W
dWishart(W, nu = p + 5, Sigma = Sigma, logarithm = TRUE)

}
